/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/automaton.h"           */
/*  Constants and definitions for automatons handling.                       */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 12/09/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef AUTOMATON_H                    /* If this file is not already        */
#define AUTOMATON_H                    /* included, include it.              */


/*
    1    Defines
    ____________
  

         Ce fichier contient les definitions des numeros d'automates, ainsi que
celles des numeros d'evenements. On a actuellement deux "tranches" de numeros
d'automates :

   - Les numeros 0 a 127 sont ceux des automates du VMOS, c'est-a-dire des
     automates executes par "moteur.c"

   - Les numeros 128 a 255 sont ceux des automates du VMIO, c'est-a-dire des
     automates executes par "motrit.c". L'automate numero 128 est TOUJOURS
     et ETERNELLEMENT reserve a AUT_RESOLVE.


         Les numeros des automates du VMOS servent d'indice dans les  trois
tables "anom", "enom" et "nnom" situees dans vmos/vmk2.c. A chaque fois que
l'on ajoute, retire un automate, ou que l'on change les valeurs des numeros, il
faut modifier ces trois tables en consequence. Si on ne le fait pas, non
seulement les traces du moteur sont fausses, mais en plus on peut generer des
ecrasements memoire. Les "tranches" de numeros d'evenements sont les suivantes:

   - Les evenements de 0 a 299 sont des codes utilisables en PanTalk, ils
     possedent des menemoniques. Ces codes sont definis dans "engine.h". On
     y trouve les codes "generaux" du VMOS, ainsi que les codes "generaux" du
     VMIO.

   - Les evenements de 300 a 499 sont les codes "generaux" du VMIO. Ils sont
     definis dans "engine.h". Ils possedent des mnemoniques, aux cas ou
     certains "remontent" jusqu'au VMOS. Mais la majorite de ces mnemoniques
     ne seront jamais utilises.

   - Les evenements a partir de 1024 sont les codes d'evenements "prives" des
     automates du VMOS. Ces codes sont definis dans le present fichier, ce qui
     signifie qu'ils ne sont pas des mnemoniques Pantalk, comme le sont les
     code de "engine.h". Par convention, on reserve pour chaque automate
     du VMOS une trance de 32 numeros consecutifs. La premiere tranche ainsi
     reservee est la tranche 1024 a 1024+31 = 1055, et est attribuee a 
     l'automate "script" AUTSCR_EXE (0)

   - Les evenements de code superieur ou egal a 10000 sont reserves aux 
     applications PanTalk. Les codes d'evenements etant des USHORT, la plus
     grande valeur est 32767. Les codes negatifs sont interdits.

*** Les codes des evenements TO_ devraient etre des codes "locaux, definis
    dans "automaton.h" et pas ici.

***  PENSEZ A TENIR A JOUR VMK2.C
*/

#define ALL_EVENT           -1         /* N'importe quel evenement           */


/*****************************************************************************
 * Numeros des automates du VMOS                                             *
 * -----------------------------                                             *
 *                                                                           *
 * Ces numeros d'automates sont compris entre 0 et 127                       *
 *                                                                           *
 * Pour chaque numero d'automate, on trouve a la suite les codes de ses      *
 * evenements "prives". Le premier de ces codes est toujours egal a          *
 * 1024 + multiple de 32. De ce fait, on dispose toujours d'au moins 32      *
 * codes par automates. Ceci n'interdit pas que certains automates puissent  *
 * utiliser deux tranches consecutives de 32.                                *
 * Les deux premiers automates n'ont pas besoin de codes specifiques, mais   *
 * cependant on reserve 32 codes (1024 a 1051) pour ces deux automates, et   *
 * dont le premier code de l'automate 2 est 1024+32 = 1056                   *
 *****************************************************************************/


/*****************************************************************************
 * Procedure "timerto" : Pour permettre la purge sans risque d'erreur des    *
 * TO, une valeur particuliere des champs "emet" des evenements est reservee *
 * aux TO du VMK. Il s'agit de la valeur 0.                                  *
 *****************************************************************************/

#define TIMERTO               0        /* vmos/sys/vmk1.c, timerto          */


/* Automate allocation buffers telecom ......................................*/

#define AUT_MEMBUF            1        /* vmos/sys/vmk1.c, membuf            */

/* Automate Interpreteur de Scripts .........................................*/

#define AUT_SCREXE            2        /* src/executif/scr_auto.c            */
#define AUTSCR_EXE    AUT_SCREXE       /* To be deleted                      */
                                       /* Evenements prives:                 */
#define SCR_EXE_CONT       1056        /* code evt script -> execution       */
#define SCR_EXE_BREAK      1057        /* code evt script -> interruption    */
#define CND_EXE_LOAD       1058        /* condition script -> chargement     */
#define SCR_EXE_SCHED      1059        /* code evt script -> scheduler       */


/* Automates claviers ...................................................... */

#define AUTTKBD               3        /* src/moteur/autkbd.c                */
                                       /* Evenements prives 1024+2*32        */
#define KEY_TEMPO          1088        /* Evt de temporisation.              */
                                       /*                                    */
#define AUTKBD                4        /* src/moteur/autkbd.c                */
                                       /* Evenements prives 1024+3*32        */
#define KEY_RELEASE_R      1120        /* Touche relachee                    */


/* Automate de gestion des evenements souris ................................*/

#define AUTMOUSE              5        /* src/moteur/autmse.c                */
                                       /* Evenements prives 1024+4*32        */
#define MOUSE_TEMPO        1152        /* Evt fin de tempo.                  */
#define MOUSE_INIT         1153        /* Evt 1er pression sur bouton souris */
#define MOUSE_STOP         1154        /* Evt retour a l'etat 0.             */
#define MOUSE_BREAK        1155        /* Evt attente de fin de move_cursor()*/
#define MOUSE_RETURN       1156        /* Evt fin du move_cursor().          */
 

/* Automate TIME ............................................................*/

#define AUT_TIME              6        /* src/moteur/auttime.c               */
                                       /* Evenements prives 1024+5*32        */
#define TIME_WATCHDOG      1184        /*                                    */


/* Automate de gestion du chargement ........................................*/

#define AUTHLOAD              7        /* src/memoire/setelt.c               */
                                       /* Evenements prives 1024+6*32        */
#define HLOAD_LOAD         1216        /* Lancer      le chargement          */
#define HLOAD_DOWNLOAD     1217        /* Attendre    le telechargement      */
#define HLOAD_SOLVE_REF    1218        /* Attendre    le chargement des ref. */
#define HLOAD_INIT         1219        /* Attendre    l'initialisation       */
#define HLOAD_WAIT_LOAD    1220        /* Attendre    le passage a utilisable*/
#define HLOAD_ERROR        1221        /* Erreur      de chargement          */
#define HLOAD_CONT         1222        /* Continuer   le chargement          */
#define HLOAD_WAIT         1223        /* Interrompre le chargement          */
#define HLOAD_CHECK_EXE    1224        /* Verifier    si executable          */


/* Automate de gestion des telechargements ..................................*/

#define AUT_DOWNLOAD          8        /* src/file/download.c                */
                                       /* Evenements prives 1024+7*32        */
#define BLOC_RECEIVED      1248        /* Bloc recu                          */
#define VOLUME_CHANGE      1249        /* Volume a change                    */
#define VOLUME_END         1250        /* Volume n'est plus diffuse          */
#define DOWNLOAD_ERROR     1251        /* Erreur de chargement               */
#define DOWNLOAD_CONT      1252        /* Continuer un autre chargement      */


/* Automate telechargement MHEG5, base sur DSMCC ............................*/

#define AUT_DSMCC_MPEG2       9        /* src/file/dsmcc.c                   */
                                       /* Evenements prives 1024+8*32        */
#define MHEG_TIMEOUT       1280        /* Fin de tempo sur reception         */
#define MHEG_ERROR         1281        /* Erreur dans ce qui a ete recu      */
#define MHEG_FIL_FOUND     1282        /* Fin de la recherche                */
#define MHEG_DII_FOUND     1283        /* Fin de demande de DII              */
#define MHEG_WAIT          1284        /* Attente de fin d'un autre chargt   */
#define DSMCC_SETDMX       1285        /* Commande de configuartion demux.   */
#define C_DSMCC_LIB        1286        /* Condition: liberer le filtre demux */
#define C_DSMCC_ERROR      1287        /* Condition: erreur allocation filtre*/
                                       /*                                    */

/* Automates DF ............................................................ */

#define AUT_DF               10        /* src/file/df.c                      */


/* Automate AUT_TASK ....................................................... */

#define AUT_TASK             11        /* vmos/moteur/vmk.c                  */
                                       /* Evenements prives 1024+10*32       */
#define RUN_TASK           1344        /* Lance une tache.                   */
#define KILL_TASK          1345        /* Tue   une tache.                   */
#define CLOSE_TASK         1346        /* Fin d'une tache.                   */
#define TASK_TIMER         1347        /* Echeance d'un timer de tache.      */
#define TASK_SIGNAL        1348        /* Un semaphore devient disponible.   */
#define TASK_EVT0          1349        /* 1er   evenement inter-taches.      */
#define TASK_EVT1          1350        /* 2nd   evenement inter-taches.      */
#define TASK_EVT2          1351        /* 3eme  evenement inter-taches.      */
#define TASK_EVT3          1352        /* 4eme  evenement inter-taches.      */
#define TASK_EVT4          1353        /* 5eme  evenement inter-taches.      */
#define TASK_EVT5          1354        /* 6eme  evenement inter-taches.      */
#define TASK_EVT6          1355        /* 7eme  evenement inter-taches.      */
#define TASK_EVT7          1356        /* 8eme  evenement inter-taches.      */
#define TASK_EVT8          1357        /* 9eme  evenement inter-taches.      */
#define TASK_EVT9          1358        /* 10eme evenement inter-taches.      */
#define TASK_EVT10         1359        /* 11eme evenement inter-taches.      */
#define TASK_EVT11         1360        /* 12eme evenement inter-taches.      */
#define TASK_EVT12         1361        /* 13eme evenement inter-taches.      */
#define TASK_EVT13         1362        /* 14eme evenement inter-taches.      */
#define TASK_EVT14         1363        /* 15eme evenement inter-taches.      */
#define TASK_EVT15         1364        /* 16eme evenement inter-taches.      */
#define TASK_EVT16         1365        /* 17eme evenement inter-taches.      */
#define TASK_EVT17         1366        /* 18eme evenement inter-taches.      */
#define TASK_EVT18         1367        /* 19eme evenement inter-taches.      */
#define TASK_EVT19         1368        /* 20eme evenement inter-taches.      */
#define TASK_EVT20         1369        /* 21eme evenement inter-taches.      */
#define TASK_EVT21         1370        /* 22eme evenement inter-taches.      */
#define TASK_EVT22         1371        /* 23eme evenement inter-taches.      */
#define TASK_EVT23         1372        /* 24eme evenement inter-taches.      */
#define TASK_EVT24         1373        /* 25eme evenement inter-taches.      */
#define TASK_EVT25         1374        /* 26eme evenement inter-taches.      */
#define TASK_EVT26         1375        /* 27eme evenement inter-taches.      */
#define TASK_EVT27         1376        /* 28eme evenement inter-taches.      */
#define TASK_EVT28         1377        /* 29eme evenement inter-taches.      */
#define TASK_EVT29         1378        /* 30eme evenement inter-taches.      */
#define TASK_EVT30         1379        /* 31eme evenement inter-taches.      */
#define TASK_EVT31         1380        /* 32eme evenement inter-taches.      */


/* Automate de gestion de la tables des ON en V3 ............................*/

#define AUT_PTLKV3           12        /* numero automate                    */
                                       /* Evenements prives 1024+12*32       */
#define PTLKV3_OFF         1408        /* Effacement d'un module             */


/* Automate  AUT_DVBSTL des sous-titres DVB .................................*/

#define AUT_DVBSTL           13        /* src/ttx/dvbstl.c                   */
                                       /* Evenements prives 1024+13*32       */
#define DVBSTL_START       1440        /* Evenement de relance.              */

/* Automate de selection de slot de controle d'acces. .......................*/

#define AUT_CA_API           14        /* src/acces/ac_api.c                 */
                                       /* Evenements prives 1024+14*32       */
#define CA_STM_ON          1472        /* Debut d'utilisation du stream.     */
#define CA_STM_UPD         1473        /* Modification de la PMT.            */
#define CA_STM_OFF         1474        /* Fin d'utilisation du stream.       */
#define CA_STM_STAT        1475        /* Nouveau statut du stream pour le CA*/
#define CA_ACTIVE          1476        /* Module CA externe insere/CA actif. */
#define CA_INACTIVE        1477        /* Module CA extrn retire/CA desactive*/
#define CA_SC_INIT         1478        /* Initialisation d'1 lecteur de carte*/
#define CA_SC_IN           1479        /* Smart Card reconnue (slot CA).     */
#define CA_STM_CLEAR       1480        /* Condition: stream non crypte.      */
#define CA_STM_CRYPT       1481        /* Condition: pas de CA pour decrypter*/
#define CA_USE_EMB         1482        /* Condition: choix d'1 CA embarque.  */
#define CA_USE_UC          1483        /* Condition: choix d'1 CA emb UC.    */
#define CA_USE_EXT         1484        /* Condition: choix d'1 CA externe.   */
#define CA_USE_DEF         1485        /* Condition: choix d'1 CA par defaut.*/
#define CA_STILL           1486        /* Condition: rester dans le meme etat*/
#define CA_POL_NEXT        1487        /* Temporisation de poling des slots. */
#define CA_FORCE_SEL       1488        /* Force a refaire la selection.      */
#define CA_STM_WAIT        1489        /* Attendre encore le status du stream*/
#define CA_STM_TO          1490        /* Fin d'attente de status.           */
#define CA_DEC_TO          1491        /* Fin d'attente de status d'1 decoder*/
#define CA_ASK_DEC         1492        /* Cnd: Attendre le status d'1 decoder*/
#define CA_SC_EVT          1493        /* Cnd: Carte a puce inseree (driver).*/
#define CA_TIMER           1494        /* Timer specifique d'un slot CA.     */
#define CA_SC_CLOSE        1495        /* Arret d'un slot de carte (standby).*/


/* Automate driver pour Viaccess ............................................*/

#define AUT_VA_DRV           15        /* src/acces/va_drv.c                 */
                                       /* Evenements prives 1024+15*32       */
#define VA_DMX_ALLOC       1504        /* Requete d'allocation d'1 scnl demux*/
#define VA_DMX_SET         1505        /* Requete de configuration d'1 filtre*/
#define VA_DMX_START       1506        /* Requete de demarrage d'1 filtre.   */
#define VA_DMX_STOP        1507        /* Requete d'arret d'1 filtre.        */
#define VA_DMX_FREE        1508        /* Requete de liberation d'1 filtre.  */
#define VA_DMX_SSF         1509        /* Requete de config filtres softwares*/
#define VA_KEY_SET         1510        /* Requete de mise en place de cles.  */
#define VA_KEY_DEL         1511        /* Requete de suppressions de cles.   */
#define VA_SMC_ON          1512        /* Requete de demarrage               */
#define VA_SMC_OFF         1513        /* Requete d'arret                    */
#define VA_SMC_RESET       1514        /* Requete de reset de la carte a puce*/
#define VA_SMC_SEND        1515        /* Requete d'envoi de commande        */
#define VA_SMC_ACT         1516        /* Requete d'activation               */
#define VA_SMC_DEACT       1517        /* Requete de desactivation           */
#define VA_SMC_CLOSE       1518        /* Requete de fermeture               */
#define VA_SEC_INFO        1519        /* Requete de lecture UID et Fuse CWK.*/
#define VA_SEC_MODE        1520        /* Requete de config mode CW.         */
#define VA_SEC_FUSE        1521        /* Requete de fusing CWK.             */
#define VA_SEC_CWK         1522        /* Requete de config CWK.             */
#define VA_SEC_DMA         1523        /* Requete de transfert Secure DMA.   */
#define C_VA_ERROR         1524        /* Cnd: Echec d'une requete.          */
#define C_VA_ERROR2        1525        /* Cnd: Echec d'une requete.          */
#define C_VA_STILL         1526        /* Cnd: Rester dans le meme etat.     */
#define C_VA_SET           1527        /* Cnd: Configurer.                   */
#define C_VA_START         1528        /* Cnd: Demarrer.                     */
#define C_VA_STOP          1529        /* Cnd: Arreter.                      */
#define C_VA_FREE          1530        /* Cnd: Liberer.                      */
#define C_VA_RESET         1531        /* Cnd: Resetter la carte a puce      */
#define C_VA_EMPTY         1532        /* Cnd: Pas/plus de carte.            */
#define C_VA_PTS           1533        /* Cnd: Carte prete pour PTS/T0/T1.   */
#define C_VA_T0            1534        /* Cnd: Carte prete pour T0.          */
#define C_VA_T1            1535        /* Cnd: Carte prete pour T1.          */
#define C_VA_MUTE          1536        /* Cnd: Carte ne repond pas/plus.     */
#define C_VA_SSF           1537        /* Cnd: Reconfig du SSF (filtre SW)   */


/* Automate du VM CAS .......................................................*/

#define AUT_VMCAS            16        /* src/acces/vmcas.c                  */
                                       /* Evenements prives 1024+17*32       */
#define VMC_REQ_INIT       1568        /* Requete d'initialisation de VL.    */
#define VMC_REQ_END        1569        /* Requete d'arret de VL.             */
#define VMC_REQ_WRITE      1570        /* Requete d'ecriture.                */
#define VMC_REQ_SETVAL     1571        /* Requete de configuration           */
#define VMC_RESP_INIT      1572        /* Fin d'initialisation de VL.        */
#define VMC_RESP_END       1573        /* Fin d'arret de VL.                 */
#define VMC_TO             1574        /* Fin de temporisation.              */
#define VMC_DATA           1575        /* Nouvelles donnees recues.          */
#define C_VMC_ERR          1576        /* Cnd: Erreur.                       */
#define C_VMC_ERR2         1577        /* Cnd: Erreur.                       */
#define C_VMC_OK           1578        /* Cnd: Succes.                       */
#define C_VMC_READY        1579        /* Cnd: On a fini.                    */
#define C_VMC_NEXT         1580        /* Cnd: Succes, etape suivante.       */
#define C_VMC_END          1581        /* Cnd: Arret a effectuer.            */


/* Automate demux/carte/vnm  pour Aladin ....................................*/

#define AUT_ALA_DMX          17        /* scr/acces/ala_dmx.c                */
                                       /* Evenements prives 1024+18*32       */
#define ALA_DMX_START      1600        /* L'acquisition est lancee.          */
#define ALA_DMX_STOP       1601        /* L'acquisition est arretee.         */
#define ALA_DMX_TO         1602        /* Fin de tempo attente d'acquisition.*/
#define ALA_DMX_STILL      1603        /* Condition: rester dans etat courant*/
                                       /*                                    */
#define AUT_ALA_SC           18        /* src/acces/ala_smc.c                */
                                       /* Evenements prives 1024+19*32       */
#define ALA_SMC_OFF        1632        /* Acces au slot n'est plus autorise. */
#define ALA_SMC_ON         1633        /* Acces au slot est autorise.        */
#define ALA_SMC_REQ        1634        /* Une requete io est lancee.         */
#define ALA_SMC_RST        1635        /* Un reset a ete demande.            */
#define ALA_SMC_MOD        1636        /* Aladin demande mode=ACCESS_NONE.   */
#define ALA_SMC_STILL      1637        /* Cnd: rester dans l'etat courant.   */
#define ALA_SMC_WERR       1638        /* Cnd: erreur en emission.           */
#define ALA_SMC_PTS        1639        /* Cnd: envoyer une requete PTS.      */
#define ALA_SMC_RWRT       1640        /* Cnd: driver tente de reemettre.    */
                                       /*                                    */
#define AUT_ALA_NVM          19        /* src/acces/ala_nvm.c                */
                                       /* Evenements prives 1024+22*31       */


/* Automate demux/carte pour Conax ..........................................*/

#define AUT_CNX_SMC          20        /* src/acces/cnx_smc.c                */
                                       /* Evenements prives 1024+21*32       */
#define CNX_SMC_OFF        1696        /* Acces au slot n'est plus autorise. */
#define CNX_SMC_ON         1697        /* Acces au slot est autorise.        */
#define CNX_SMC_REQ        1698        /* Une requete io est lancee.         */
#define CNX_SMC_RST        1699        /* Un reset a ete demande.            */
#define CNX_SMC_STILL      1700        /* Cnd: rester dans l'etat courant.   */
#define CNX_SMC_WERR       1701        /* Cnd: erreur en emission.           */
#define CNX_SMC_RERR       1702        /* Cnd: erreur en reception.          */
#define CNX_SMC_DAT        1703        /* Cnd: donnees a lire dans la carte. */
                                       /*                                    */
#define AUT_CNX_DMX          21        /* src/acces/cnx_dmx.c                */
                                       /* Evenements prives 1024+22*32       */

/* Automate de gestion de requetes pour Conax ...............................*/

#define AUT_CNX_REQ          22        /* src/acces/cnx_req.c                */
                                       /* Evenements prives 1024+23*32       */
#define CNX_REQ_EXT        1760        /* Extraction de la carte a puce.     */
#define CNX_REQ_INS        1761        /* Insertion d'une carte a puce.      */
#define CNX_REQ_RST        1762        /* Fin de reset de la carte a puce.   */
#define CNX_REQ_END        1763        /* Fin de la requete courante.        */
#define CNX_REQ_INIT_SMC   1764        /* Requete d'initialisation d'une smc.*/
#define CNX_REQ_SEND_CAT   1765        /* Requete d'envoi de la CAT.         */
#define CNX_REQ_SEND_PMT   1766        /* Requete d'envoi de la PMT.         */
#define CNX_REQ_SEND_EMM   1767        /* Requete d'envoi d'un EMM.          */
#define CNX_REQ_SEND_ECM   1768        /* Requete d'envoi d'un ECM.          */
#define CNX_REQ_SEND_STR   1769        /* Requete d'envoi reponse a enquiry. */
#define CNX_REQ_SEND_CHR   1770        /* Requete d'envoi d'un caractere.    */
#define CNX_REQ_OA_SEL     1771        /* Requete d'envoi d'une selection OA.*/
#define CNX_REQ_RELOCK     1772        /* Requete reverrouillage maturity/PPV*/
#define CNX_REQ_CONSULT    1773        /* Requete de consultation de la carte*/
#define CNX_REQ_PINCOM     1774        /* Requete concernant le PIN Code.    */
#define CNX_REQ_RFD        1775        /* Refus de la carte a puce.          */
#define CNX_REQ_STILL      1776        /* Condition: rester dans le meme etat*/
#define CNX_REQ_REINIT     1777        /* Condition: reset de la carte.      */
#define CNX_REQ_ERR        1778        /* Condition: erreur detectee.        */
#define CNX_REQ_RPH        1779        /* Condition: requete vers Return Path*/
#define CNX_REQ_INISESS    1780        /* Condition: initialiser les sessions*/
#define CNX_REQ_READY      1781        /* Condition: carte prete.            */


/* Automate Host pour Conax .................................................*/

#define AUT_CNX_HOST         23        /*                                    */
                                       /* Evenements prives 1024+24*32       */
#define CNX_HOST_EXT       1792        /* Extraction de la carte a puce.     */
#define CNX_HOST_INS       1793        /* Carte inseree, initialisee et prete*/
#define CNX_HOST_END       1794        /* Fin de la requete courante.        */
#define CNX_HOST_EMM       1795        /* Reception d'un EMM.                */
#define CNX_HOST_ECM       1796        /* Reception d'un ECM.                */
#define CNX_HOST_CAT       1797        /* Transmettre la CAT.                */
#define CNX_HOST_PMT       1798        /* Transmettre la PMT.                */
#define CNX_HOST_CHR       1799        /* Transmettre un choix applicatif.   */
#define CNX_HOST_STR       1800        /* Transmettre une saisie applicative.*/
#define CNX_HOST_RELOCK    1801        /* Transmettre demande reverrouillage.*/
#define CNX_HOST_CONSULT   1802        /* Transmettre requete de consultation*/
#define CNX_HOST_PINCOM    1803        /* Transmettre requete liee o PIN Code*/
#define CNX_HOST_ERR       1804        /* Condition: Pb SMC irrecuperable.   */
#define CNX_HOST_WAIT      1805        /* Condition: aller attendre requete. */
#define CNX_HOST_REINIT    1806        /* Condition: reinitialisation smc.   */


/* Automate Mediaguard ......................................................*/

#define AUT_MG_DRV           24        /*                                    */
                                       /* Evenements prives 1024+25*32       */
#define MG_DMX_CBK         1824        /* Evt: Appeler une callback MG.      */
#define MG_DMX_TABLE       1825        /* Evt: Reception d'une table DVB.    */
#define MG_KEY_SET         1826        /* Evt: Modifier les cles de dechiffrt*/
#define MG_FLT_START       1827        /* Evt: Requete MG de demarrage filtre*/
#define MG_FLT_STOP        1828        /* Evt: Requete MG d'arret d'un filtre*/
#define MG_FLT_TO          1829        /* Evt: Time-out d'attente de section.*/
#define MG_FLT_NONE        1830        /* Cnd: Echec d'allocation de filtre. */
#define MG_FLT_FREE        1831        /* Cnd: Liberer le filtre.            */
#define MG_FLT_ALLOC       1832        /* Cnd: Reallouer le filtre.          */
#define MG_FLT_CFG         1833        /* Cnd: Reconfigurer le filtre.       */
                                       /* Carte a puce pour Mediaguard.      */
#define MG_SMC_ON          1834        /* Acces au slot est autorise.        */
#define MG_SMC_OFF         1835        /* Acces au slot n'est plus autorise. */
#define MG_SMC_CLOSE       1836        /* Fermeture du slot (drv_close).     */
#define MG_SMC_RESET       1837        /* Requete de reset du slot.          */
#define MG_SMC_LOCK        1838        /* Requete de verrouillage du slot.   */
#define MG_SMC_UNLOCK      1839        /* Suppression du verrouillage du slot*/
#define MG_SMC_LOCKTO      1840        /* Time-out verrouillage du slot.     */
#define MG_SMC_PTS         1841        /* Requete PTS pour le slot.          */
#define MG_SMC_SEND        1842        /* Requete d'emission pour le slot.   */
#define MG_SMC_SUBSCR      1843        /* Requete d'enregistrement call-back.*/
                                       /* Commentaire                        */
#define C_MG_RESET         1844        /* Cnd: Reset avant PTS pour le slot. */
#define C_MG_OK            1845        /* Cnd: Succes d'un PTS sans negocier.*/
#define C_MG_KO            1846        /* Cnd: Echec d'une requete.          */
#define C_MG_EMPTY         1847        /* Cnd: Pas/plus de carte.            */
#define C_MG_PTS           1848        /* Cnd: Carte prete pour PTS/T0/T1.   */
#define C_MG_T0            1849        /* Cnd: Carte prete pour T0.          */
#define C_MG_T1            1850        /* Cnd: Carte prete pour T1.          */
#define C_MG_MUTE          1851        /* Cnd: Carte ne repond pas/plus.     */
#define C_MG_ERROR         1852        /* Cnd: Erreur de communication smc.  */
#define C_MG_IDLE          1853        /* Cnd: Rester dans le meme etat.     */
                                       /* Evenements Non Volatile Memory     */
#define MG_NVM_WRT         1854        /* Requete MG d'ecriture en NVM.      */
                                       /* Evenements OS                      */
#define MG_OS_TIM          1855        /* Evt: echeance d'un timer MG.       */
#define MG_OS_INHIBIT      1856        /* Evt: echeance d'un timer MG.       */


/* Automate CA pour Mediaguard ..............................................*/

#define AUT_MG_CA            25        /*                                    */
                                       /* Evenements prives 1024+27*32       */
#define MG_CA_MAT_TO       1888        /* Evt: echeance d'un timer MG.       */
#define MG_CA_PPV_TO       1889        /* Evt: echeance d'un timer MG.       */
#define MG_CA_TST_TO       1890        /* Evt: echeance du timer de test.    */


/* Automate CA DVB-CI .......................................................*/

#define AUT_CI_CA            26        /* "$vmos/acces/dvbci_ca.c"           */
                                       /* Evenements prives 1024+28*32       */
#define IOD_SET            1920        /* Mise en place de l'IOD protocole   */
#define IOD_UNSET          1921        /* Retirer l'IOD de protocole.        */
#define EVT_PRGON          1922        /* Demarrage du dechiffrement         */
#define EVT_PRGUPD         1923        /* La PMT du programme a change       */
#define EVT_PRGOFF         1924        /* Arret du dechiffrement d'un prog.  */
#define C_UPDATE           1925        /* Condition de mise a jour           */
#define C_CAPOS            1926        /* Condition positive de dechiffrage  */
#define C_CANEG            1927        /* Condition negative de dechiffrage  */
#define EVT_PRGADD         1928        /* Demarrage d'un second dechiffrement*/
#define C_HSNEG            1929        /* Host shunning active               */
#define C_HSPOS            1930        /* Host shunning active               */
#define CICA_TO            1931        /* Temporisateur                      */
#define EVT_SDTUPD         1932        /* Chgt SDT                           */
#define CICA_TO2           1933        /* Temporisateur                      */
#define C_CIPLUS           1934        /* Cas Ci + avec Host shunning        */
#define C_SDT_CLR          1935        /* SDT deja presente (cas clear)      */
#define C_SDT_SCR          1936        /* SDT deja presente (cas scramble)   */
#define C_CLEAR            1937        /* Programme en clair                 */
#define C_SCRAMBLED        1938        /* Programme scramble                 */
#define HS_STEP            1939        /* Host shunning                      */
#define HS_TPCHG           1940        /* Changement de TP, SDT a recuperer  */
#define C_RESTART          1941        /* Traitement de restart, cas CI+     */


/* Automate Teletexte .......................................................*/

#define AUT_TTXT             27        /* Teletexte                          */
                                       /* Evenements prives 1024+29*32       */
#define TTXT_SELECT        1952        /* Selection d'une page pour affichage*/


/* Automate telechargement DSMCC ............................................*/

#define AUT_DSM_FILE         28        /* "$vmos/file/dsmcc.c"               */
                                       /* Evenements prives 1024+30*32       */

/* Automate chargement de librairie dynamique ...............................*/

#define AUT_DL               29        /* "$vmos/divers/elf_dl.c"            */
                                       /* Evenements prives 1024+31*32       */
#define DL_READ_SYS        2016        /* Debut lecture zone systeme NAND    */
#define DL_READ_LIB        2017        /* Commande de chargement d'une lib.  */
#define DL_FREE_LIB        2018        /* Commande de dechargement d'une lib.*/
#define DL_INIT_DONE       2019        /* Evt de fin d'init systeme.         */
#define DL_TO_INIT         2020        /* Temporisations de initialisations. */
#define DL_TO_FINI         2021        /* Temporisations de terminaisons.    */
#define DL_TO_MD5          2022        /* Temporisation de verification MD5. */
#define DL_TO_BSS          2023        /* Temporisation d'installation BSS.  */
#define DL_TO_RELOC        2024        /* Temporisation de relocation.       */
#define DL_C_READY         2025        /* Cnd: fin d'etape.                  */
#define DL_C_DATA          2026        /* Cnd: fichier de donnees disponible.*/
#define DL_C_ERROR         2027        /* Cnd: erreur de lecture.            */
#define DL_C_ABORT         2028        /* Cnd: arret du chargement.          */
#define DL_C_WAIT          2029        /* Cnd: attendre avant de continuer.  */
#define DL_C_DONE          2030        /* Cnd: fin d'execution de la commande*/
#define DL_C_MD5           2031        /* Cnd: debut de verification MD5.    */
#define DL_C_BSS           2032        /* Cnd: debut de l'installation du BSS*/
#define DL_C_RELOC         2033        /* Cnd: debuter les relocations.      */
#define DL_C_INIT          2034        /* Cnd: debuter les initialisations.  */
#define DL_C_FREED         2035        /* Cnd: fin de liberation d'une lib.  */


/* Automate du service OAD ..................................................*/

#define AUT_SRV_OAD          30        /* "$vmos/telecom/srv_oad.c"          */
                                       /* Evenements prives 1024+32*32       */
#define OAD_START          2048        /* Debut de recherche OAD.            */
#define OAD_STOP           2049        /* Arret forcee de recherche OAD.     */
#define OAD_TO             2050        /* Timeout de telechargement OAD.     */
#define C_OAD_SBOTH        2051        /* Cnd: charger PAT + (NIT ou BAT).   */
#define C_OAD_SPAT         2052        /* Cnd: charger PAT.                  */
#define C_OAD_SNIT         2053        /* Cnd: charger NIT.                  */
#define C_OAD_SBAT         2054        /* Cnd: charger BAT.                  */
#define C_OAD_SPMT         2055        /* Cnd: charger PMT.                  */
#define C_OAD_SUNT         2056        /* Cnd: charger UNT.                  */
#define C_OAD_SDSI         2057        /* Cnd: charger DSI.                  */
#define C_OAD_SDII         2058        /* Cnd: charger DII.                  */
#define C_OAD_SSC0         2059        /* Cnd: charger Section 0.            */
#define C_OAD_SB0          2060        /* Cnd: charger Bloc 0 du volume.     */
#define C_OAD_WBLK         2061        /* Cnd: charger autres blocs du vol.  */
#define C_OAD_WWRT         2062        /* Cnd: attendre fin d'ecriture.      */
#define C_OAD_FOUND_I      2063        /* Cnd: on a trouve, IDLE.            */
#define C_OAD_FOUND_S      2064        /* Cnd: on a trouve, STOP.            */
#define C_OAD_FAILED_S     2065        /* Cnd: echec/erreur, STOP.           */
#define C_OAD_FAILED_I     2066        /* Cnd: echec/erreur, IDLE.           */
#define C_OAD_DONE         2067        /* Cnd: telechargement du vol termine.*/
#define C_OAD_CONTINUE     2068        /* Cnd: rester dans l'etat courant.   */
#define C_OAD_GIVEN        2069        /* Cnd: donnees transmises par l'appli*/


/* Nombre max d'automates ...................................................*/

/* ATTENTION Mettre la meme valeur dans NAUTMAX des fichiers *.sys.          */

#define AUTLAST              31        /* Dernier numero d'automate          */



/*****************************************************************************
 * Numeros des automates du VMIO                                             *
 * -----------------------------                                             *
 *                                                                           *
 * Ces numeros d'automates sont compris entre 128 et 255                     *
 *                                                                           *
 * Pour chaque numero d'automate, on trouve a la suite les codes de ses      *
 * evenements "prives". Le premier de ces codes est toujours egal a          *
 * 1024 + 32 * numero de l'automate. De ce fait, on dispose de 32 codes      *
 * pour chacun des automates                                                 *
 *****************************************************************************/

#define FEVT(aut) (1024+32*(aut))      /* Premier code d'evenenement dispo.  */
#define LEVT(aut) (1024+32*(aut)+31)   /* Dernier code d'evenenement dispo.  */

#define AUTS_FIRST         128         /* Numero premier automate simplifie  */

/* Automate de resolution des noms de drivers ...............................*/

#define AUT_RESOLVE        128         /* vmio/drvsys/drv_rslv.c             */

/* Automate interface clavier/telecommande ..................................*/

#define AUT_KBDITF         129         /* vmio/drvcar/drv_kbditf.c           */

/* Automate DAC .............................................................*/

#define AUT_DAC            130         /* vmio/drvcar/drv_dac.c              */

/* Automate ADC .............................................................*/

#define AUT_ADC            131         /* vmio/drvcar/drv_adc.c              */

/* Automate ASY .............................................................*/

#define AUT_ASY              135       /* vmio/drvsys/drv_asy.c              */

#define REQ_IO              5344       /* Requete de relance req_lib.c       */
#define IT_SIG              5345       /* Changement d'etat signal de jnctin */

#define TO_SIG0             5346       /* Tempo file d'ecriture              */
#define TO_SIG1             5347       /* Tempo file de lecture              */
#define TO_SIG2             5348       /* Tempo file autre                   */
#define TO_MSEC0            5349       /* Tempo file d'ecriture              */
#define TO_MSEC1            5350       /* Tempo file de lecture              */
#define TO_MSEC2            5351       /* Tempo file autre                   */

#if REQ_IO != FEVT(AUT_ASY)
  #error "AUT_ASY first event has wrong number !"
#endif

#if TO_MSEC2 > LEVT(AUT_ASY)
  #error "AUT_ASY defines too many events !"
#endif

/* Automate SRF .............................................................*/

#define AUT_SRF            136         /* vmio/drvcar/drv_srf.c              */

/* Automate BLT .............................................................*/

#define AUT_BLT            137         /* vmio/drvcar/drv_blt.c              */

/* Automate SPI .............................................................*/

#define AUT_SPI            138         /* vmio/drvsys/drv_spi.c              */

/* Automate GPIO ............................................................*/

#define AUT_GPIO           139         /* vmio/drvsys/drv_gpio.c             */

/* Automate LED .............................................................*/

#define AUT_LED            140         /* vmio/drvcar/drv_led.c              */

/* Automate protocoles infra-rouge (RCU) ....................................*/

#define AUT_RCU            141         /* vmio/drvsys/drv_rcu.c              */

#define CR_C               5536        /* Cond Raw symbols capture           */
#define CT_C               5537        /* Cond Raw symbols capture           */
#define CR_S               5538        /* Cond symbols capture               */
#define CT_S               5539        /* Cond symbols capture               */
#define CR_N               5540        /* Cond telecommande a leader/tail    */
#define CT_N               5541        /* Cond telecommande a leader/tail    */
#define CR_R               5542        /* Cond telecommande a repetition mess*/
#define CT_R               5543        /* Cond telecommande a repetition mess*/
#define CR_6               5544        /* Cond telecommande RC6 mode 0       */
#define CT_6               5545        /* Cond telecommande RC6 mode 0       */
#define CR_5               5546        /* Cond telecommande RC5              */
#define CT_5               5547        /* Cond telecommande RC5              */
#define CR_X               5548        /* Cond protocole XMP                 */
#define CT_X               5549        /* Cond protocole XMP                 */
#define CR_M               5550        /* Cond protocole RCMM                */
#define CT_M               5551        /* Cond protocole RCMM                */
#define CR_H               5552        /* Cond protocole HomeEasy (32 bits)  */
#define CT_H               5553        /* Cond protocole HomeEasy (32 bits)  */
#define CR_L               5554        /* Cond protocole HomeEasy (36 bits)  */
#define CT_L               5555        /* Cond protocole HomeEasy (36 bits)  */
#define CR_D               5556        /* Cond signal DCF77                  */
#define CT_D               5557        /* Cond signal DCF77                  */
#define C_EVT              5558        /* Cond evt a remonter au VMOS        */
#define C_NOEVT            5559        /* Cond pas d'evt a remonter au VMOS  */
#define C_RCU              5560        /* Cond message RCU                   */
#define C_KBD              5561        /* Cond message Clavier               */
#define C_MSE              5562        /* Cond message Souris                */
#define C_PFIN             5562        /* Cond prochain symbole est STOP     */
#define C_CORR             5563        /* Cond signal correle                */
#define C_SYNC             5564        /* Cond reception synchonisee         */

/* Automate protocole I2C ...................................................*/

#define AUT_I2C            142         /* vmio/drvsys/drv_i2c.c              */

#if CR_C != FEVT(AUT_RCU)
  #error "AUT_RCU first event has wrong number !"
#endif

#if C_ES > LEVT(AUT_RCU)
  #error "AUT_RCU defines too many events !"
#endif

/* Automate driver EEPROM ...................................................*/

#define AUT_EEP            143         /* vmio/drvsys/drv_eep.cc             */

/* Automate ETHERNET ........................................................*/

#define AUT_ETH            144         /* vmio/drvcar/drv_eth.c              */
                                       /*                                    */
#define IT_STA             5632        /* Changement d'etat PHY status       */
#define TO_STA             5633        /* Polling timeout for PHY status     */
#define TO_NEG             5634        /* End of self-negociation timeout    */

/* Automate LOOPBACK ........................................................*/

#define AUT_LO             145         /* vmio/drvcar/drv_lo.c               */

/* Automate protocole ARP ...................................................*/

#define AUT_ARP            146         /* vmio/pro/pro_arp.c                 */
                                       /*                                    */
#define TO_ARP             5696        /* Timeout de verification/resolution */

/* Automate protocole IP ....................................................*/

#define AUT_IP             147         /* vmio/pro/pro_ip.c                  */
                                       /*                                    */
#define TO_GETIP           5728        /* Condition ouverture en mode IP     */

/* Automate protocole TCP ...................................................*/

#define AUT_TCP            148         /* vmio/pro/pro_tcp.c                 */
                                       /*                                    */



/*
    2    Structures and typedefs
    ____________________________
*/


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of AUTOMATON_H.                */
