/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/dro.h"                 */
/*  Constants and definitions for drivers interface.                         */
/*                                                                           */
/*  Author : JYA                        Last modification : 09/11/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1 Constantes de definition des types de drivers
            1.2 Constantes du decoupage en bits du champ "longueur" du REQ_OPEN
         2  Structures and typedefs
            2.1 Structure de description d'un tag (ROM)
            2.2 Structure de description d'un driver systeme du VMOS (ROM)
            2.3 Structure de description d'un driver du VMOS (ROM)
            2.4 Structure d'iteration pour manipulation des listes de registres
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DRV_H                          /* If this file is not already        */
#define DRV_H                          /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    1.1 Constantes de definition des types de drivers
    -------------------------------------------------
*/


/*****************************************************************************
 * Ces constantes sont utilises pour les champs "typdrv" des structures VMOS *
 * "Sys_driver", "Dps_driver" et "Df_driver".                                *
 * Elles sont egalement utilisees pour le champ "typdrv" de la structure VMIO*
 * "Drv_desc"..                                                              *
 *****************************************************************************/


#define DRV_SYS               0        /* Driver systeme                     */
#define DRV_CAR               1        /* Driver caractere                   */
#define DRV_PRO               2        /* Protocole                          */
#define DRV_SER               3        /* Service                            */
#define DRV_BLK               5        /* Driver bloc                        */
#define DRV_VOL               6        /* Driver volume                      */

#define BSYS       (1<<DRV_SYS)
#define BCAR       (1<<DRV_CAR)
#define BPRO       (1<<DRV_PRO)
#define BSER       (1<<DRV_SER)
#define BBLK       (1<<DRV_BLK)
#define BVOL       (1<<DRV_VOL)

#define BCHA       (1<<13)
#define BSUB       (1<<14)
#define BMODD      (1<<15     )

#define BCPS       (BCAR|BPRO|BSER)
#define BDPS       (BCAR|BPRO|BSER|BBLK)

#define BTMD ( BSYS | BCAR | BPRO | BSER | BBLK | BVOL )

#define BTMA ( BTMD | BMODD )

/*
    1.2 Constantes du decoupage en bits du champ "longueur" du REQ_OPEN
    -------------------------------------------------------------------
*/


#define NO_MSKR        (UCHAR)255      /* Pas de registre de masque.         */


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    2.1 Structure de description d'un tag (ROM)
    -------------------------------------------
*/


/*****************************************************************************
 * Le champ "ptag" de la structure "Dps_driver" pointe un tableau de         *
 * structures "Ptag". Ce tableau contient une structure de type "Ptag" par   *
 * TAG. Chaque structure "Ptag" decrit les proprietes d'un seul tag. Cette   *
 * structure ne contient PAS le nom du tag. La structure "Dps_driver"        *
 * contient un champ nomme "tag", qui pointe un tableau d'adresses de        *
 * chaines, ces chaines sont les noms des tags. De meme, lorsqu'un tag est   *
 * un ENUM, la liste des valeus possibles n'est PAS dans la structure "Ptag",*
 * mais est pointee par le champ "val" de la structure "Dps_driver".         *
 *****************************************************************************/


typedef struct                         /* Proprietes d'un tag                */
  BEGIN                                /* ---------------------------------- */
    UCHAR           type             ; /* Type de tag                        */
    UINT            acces            ; /* Accessibilite du tag               */
    USHORT          valr             ; /* Numero du 1er reg. pour le tag.    */
    USHORT          nbr              ; /* Nombre de registres pour le tag    */
    UCHAR           shift            ; /* Shift vers la gauche a appliquer.  */
    UINT            mask             ; /* Masque a appliquer sur valeur recue*/
    UINT            sessid_mod       ; /* Indication de changement de sessid */
  END               Ptag             ; /* ---------------------------------- */


typedef struct                         /* Pointeurs sur les infos des tags.  */
  BEGIN                                /* ---------------------------------- */
    INT             ntag             ; /* Nombre de tags.                    */
    const CHAR     *const *tag       ; /* Tableau adresses des noms des tags */
    const CHAR     *const *val       ; /* Tableau des valeurs des tags ENUM. */
    const Ptag     *ptag             ; /* Tableau des proprietes des tags.   */
  END               Drv_tag          ; /* ---------------------------------- */

typedef struct                         /* Numero de tag -> numero de reg.    */
  BEGIN                                /* ---------------------------------- */
    UINT            reg              ; /* Numero du 1er reg. pour le tag.    */
    UINT            msk              ; /* Numero du reg. de masque de ce tag.*/
  END               Tag2reg          ; /* ---------------------------------- */


/*
    2.2 Structure de description d'un driver systeme du VMOS (ROM)
    --------------------------------------------------------------
*/


/*****************************************************************************
 * Un VMOS possede un tableau nomme "vmos_driver[]", qui lui permet de       *
 * retrouver ses drivers, ses piles de protocoles et services. Le tableau    *
 * "vmos_driver" est un tableau d'adresses de structures de type             *
 * "Sys_driver", "Dps_driver" et "Df_driver". Chaque module C "driver        *
 * systeme" contient une structure "Sys_driver" dont l'adresse sera l'un des *
 * elements de "vmos_driver". L'indice dans "vmos_driver[]" est appele le    *
 * "major number".                                                           *
 *****************************************************************************/


typedef struct  sys_driver             /* Un driver systeme                  */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[STDNAME]    ; /* Nom du driver systeme              */
    CHAR            typdrv           ; /* Vaut toujours 0                    */
    IWORD           (*drv_evt   )()  ; /* Ad procedure xxx_yyy_evt           */
  END               Sys_driver       ; /* ---------------------------------- */


/*
    2.3 Structure de description d'un driver du VMOS (ROM)
    ------------------------------------------------------
*/


/*****************************************************************************
 * Chaque module C de type driver, pile de protocoles ou service contient la *
 * definition d'une structure "Dps_driver", dont l'adresse sera l'un des     *
 * elements de "vmos_driver". L'indice dans "vmos_driver[]" est appele le    *
 * "major number".                                                           *
 *****************************************************************************/

typedef struct  dps_driver             /* One VMOS driver/protocole/service  */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[STDNAME]    ; /* driver/protocole/service name      */
    CHAR            typdrv           ; /* 001:dcar 010:proto 011:ser 100:dblk*/
                                       /* Interface procedures ------------- */
    IWORD      (*drv_evt         )() ; /* xyz_evt          procedure address */
    IWORD      (*drv_info        )() ; /* xyz_info         procedure address */
    IWORD      (*drv_init        )() ; /* xyz_init         procedure address */
    IWORD      (*drv_end         )() ; /* xyz_end          procedure address */
    IWORD      (*drv_alloc       )() ; /* xyz_alloc        procedure address */
    IWORD      (*drv_free        )() ; /* xyz_free         procedure address */
    IWORD      (*drv_open_device )() ; /* xyz_open_device  procedure address */
    IWORD      (*drv_close_device)() ; /* xyz_close_device procedure address */
    IWORD      (*drv_alloc_sub   )() ; /* xyz_alloc_sub    procedure address */
    IWORD      (*drv_free_sub    )() ; /* xyz_free_sub     procedure address */
    IWORD      (*drv_connect     )() ; /* xyz_connect      procedure address */
    IWORD      (*drv_listen      )() ; /* xyz_listen       procedure address */
    IWORD      (*drv_disconnect  )() ; /* xyz_disconnect   procedure address */
    IWORD      (*drv_abort       )() ; /* xyz_abort        procedure address */
    IWORD      (*drv_write       )() ; /* xyz_write        procedure address */
    IWORD      (*drv_read        )() ; /* xyz_read         procedure address */
    IWORD      (*drv_put         )() ; /* xyz_put          procedure address */
    IWORD      (*drv_get         )() ; /* xyz_get          procedure address */
    IWORD      (*drv_setval      )() ; /* xyz_setval       procedure address */
    IWORD      (*drv_getval      )() ; /* xyz_getval       procedure address */
    IWORD      (*drv_rpc         )() ; /* xyz_rpc          procedure address */
                                       /* Tags ----------------------------- */
    INT             ntag             ; /* Count of tags                      */
    INT             ntag_sub         ; /* Count of sub-channels tags         */
    const CHAR     *const *tag       ; /* Tag names addresses array address  */
    const CHAR     *const *val       ; /* ENUM tags values array address     */
    const Ptag     *ptag             ; /* Tags properties array address      */
                                       /* Device number -------------------- */
    const UINT      nreg_idev        ; /* Register number                    */
    const UINT      sreg_idev        ; /* Right shift and then               */
    const UINT      mreg_idev        ; /* mask value                         */
                                       /* Flags ---------------------------- */
    const UCHAR     sc_size          ; /* Sub-channels number of bits        */
    UINT            alloc_flags_0    ; /* Allocation flags that must be 0    */
    UINT            alloc_flags_1    ; /* Allocation flags that must be 1    */
  END               Dps_driver       ; /* ---------------------------------- */


/*
    2.4 Structure d'iteration pour manipulation des listes de registres
    -------------------------------------------------------------------
*/


/*****************************************************************************
 * Voir drv_itf1.c section 2.13.                                             *
 *****************************************************************************/


typedef struct _s_rl_it                /*                                    */
  BEGIN                                /* ---------------------------------- */
    INT             nr               ; /* Numero de registre                 */
    INT             nbb              ; /* Nombre de blocs                    */
    INT             fmt              ; /* Format des donnees                 */
    INT             nbr              ; /* Nombre de registres dans le bloc   */
    UCHAR          *lr               ; /* Pointeur sur le bloc courant       */
    UCHAR          *ls               ; /* Pointeur sur le bloc suivant       */
    UCHAR           o[4]             ; /* 0123: Big endian, 3210: lil endian */
    UCHAR          *p                ; /* Pointeur registre dans la liste.   */
  END               Rlit             ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


IWORD drv_list_tags         (CHAR*,CHAR*,INT*,INT*                           );
IWORD drv_list_modules      (CHAR*,INT,INT*,INT,INT*                         );
IWORD drv_list_names        (CHAR*,CHAR*,INT*,INT,INT*                       );
IWORD drv_init_driver       (CHAR*,INT,INT,INT*,INT*                         );
IWORD drv_init_protocol     (CHAR*,INT,INT,INT*,INT*                         );
IWORD drv_init_service      (CHAR*,INT,INT*,INT*                             );
IWORD drv_end               (INT,INT,INT*                                    );
IWORD drv_alloc_device      (INT,CHAR*,CHAR*,CHAR*,INT,INT,INT,INT,INT*,INT* );
IWORD drv_alloc_protocol    (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*,INT*       );
IWORD drv_alloc_service     (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*,INT*       );
IWORD drv_free              (INT,INT,INT*                                    );
IWORD drv_open_device       (INT,CHAR*,INT,INT*                              );
IWORD drv_close_device      (INT,INT,INT*                                    );
IWORD drv_alloc_subchan     (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*,INT*       );
IWORD drv_free_subchan      (INT,INT,INT*                                    );
IWORD drv_connect           (INT,CHAR*,INT,INT*                              );
IWORD drv_listen            (INT,CHAR*,INT,INT*                              );
IWORD drv_disconnect        (INT,INT,INT,INT*                                );
IWORD drv_abort             (INT,INT,INT*                                    );
IWORD drv_write             (INT,UCHAR*,INT,INT,INT,INT,INT*                 );
IWORD drv_read              (INT,INT,INT,INT,UCHAR**,INT*,INT*,INT,INT*      );
IWORD drv_put               (INT,UCHAR*,UINT,UINT,INT,INT,INT,INT*           );
IWORD drv_get               (INT,UINT,UINT,INT,INT,UCHAR**,INT,INT*          );
IWORD drv_setval            (INT,CHAR*,INT,INT*                              );
IWORD drv_getval            (INT,CHAR*,INT,INT,INT*                          );
IWORD drv_getval2           (INT,CHAR*,INT,INT*                              );
IWORD drv_execute           (INT,CHAR*,INT,INT*                              );
IWORD drv_rpc               (INT,CHAR*,CHAR*,CHAR*,INT,INT,INT*              );
IWORD drv_rpc2              (INT,CHAR*,CHAR*,CHAR*,INT,INT*                  );
IWORD drv_name2major        (CHAR*,INT*,INT*                                 );
IWORD drv_chk_ioddev        (UINT,INT*,INT*,INT*,INT*                        );
IWORD drv_set_state         (S_evt*                                          );
IWORD tag2reg               (const Dps_driver*                                ,
                             INT,INT,CHAR*[4],USHORT[4],UINT*,INT,UINT*,INT* );
IWORD tag2cmd               (const Dps_driver*,INT,INT,CHAR*,UINT*,UINT*     );
IWORD reg2tag               (const Dps_driver*,INT,UINT*,UINT,CHAR*,INT,INT* );
IWORD init_telecom          (void                                            );

void  unschedule            (INT,INT,IWORD(*)(),INT,S_evt**,IWORD*           );
IWORD set_rets              (INT                                             );


#endif                                 /* End of DRV_H.                      */
