/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_adc.h"             */
/*  Constants and definitions for ADC.                                       */
/*                                                                           */
/*  Author : JYA                        Last modification : 04/27/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Register numbers
            1.2  Tag numbers and counts
            1.3  Flags for supported bit depth per sample
            1.4  IT_RX events error codes

         2  Structures and typedefs
            2.1  High level ADC driver definitions - Adc_drvdef
            2.2  Low level ADC driver interface - Adc_desc
            2.3  ADC input context - Adc_inpctx
            2.4  ADC device context - Adc_devctx

         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in board_xyz.c

*/


#ifndef DRV_ADC_H                      /* If this file is not already        */
#define DRV_ADC_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Register numbers
    ---------------------
*/

#define NR_ADC_TYPE               0    /* string  TYPE          (  1 UINT )  */
#define NR_ADC_NAME               1    /* string  NAME          (  2 UINTs)  */
#define NR_ADC_NUMDEV             3    /* integer NUM           (  1 UINT )  */

#define NR_ADC_UNAMES             4    /* string  UNAMES        (128 UINTs)  */
#define NR_ADC_VREF             132    /* integer VREF          (  1 UINT )  */
#define NR_ADC_LDEPTH           133    /* integer LDEPTH        (  1 UINT )  */
#define NR_ADC_LDEPDMA          134    /* integer LDEPTH        (  1 UINT )  */
#define NR_ADC_MINFREQ          135    /* integer MINFREQ       (  1 UINT )  */
#define NR_ADC_MAXFREQ          136    /* integer MAXFREQ       (  1 UINT )  */
#define NR_ADC_MAXFAST          137    /* integer MAXFAST       (  1 UINT )  */

#define NR_ADC_DEPTH            138    /* integer DEPTH         (  1 UINT )  */
#define NR_ADC_SCANFREQ         139    /* integer SCANFREQ      (  1 UINT )  */
#define NR_ADC_SAMPFREQ         140    /* integer SAMPFREQ      (  1 UINT )  */

#define NR_ADC_INTERLV          141    /* integer ENDIAN        (  1 UINT )  */
#define NR_ADC_REQSIZE          142    /* integer REQSIZE       (  1 UINT )  */
#define NR_ADC_REQEND           143    /* integer REQEND        (  1 UINT )  */
#define NR_ADC_OUTSIZE          144    /* integer OUTSIZE       (  1 UINT )  */
#define NR_ADC_OUTEND           145    /* integer OUTEND        (  1 UINT )  */
#define NR_ADC_OUTDMA           146    /* integer OUTDMA        (  1 UINT )  */
#define NR_ADC_ADCBITS          147    /* integer ADCBITS       (  1 UINT )  */
#define NR_ADC_LOSSCNT          148    /* integer LOSSCNT       (  1 UINT )  */

#define NR_ADC_UNAME            149    /* string  UNAME         (  2 UINT )  */
#define NR_ADC_FASTCAP          151    /* integer FASTCAP       (  1 UINT )  */
#define NR_ADC_GAINCAP          152    /* integer GAIN CAP      (  1 UINT )  */

#define NR_ADC_MODE             153    /* integer MODE          (  1 UINT )  */
#define NR_ADC_IHYST            154    /* integer IHYST         (  1 UINT )  */
#define NR_ADC_GAIN             155    /* integer GAIN          (  1 UINT )  */

#define NR_ADC_SIGN             156    /* integer SIGN          (  1 UINT )  */
#define NR_ADC_SCALING          157    /* integer SCALING       (  1 UINT )  */
#define NR_ADC_SCAMUL           158    /* integer SCAMUL        (  1 UINT )  */
#define NR_ADC_SCADIV           159    /* integer SCADIV        (  1 UINT )  */

#define NR_ADC_SAMPLE           160    /* integer SAMPLE        (  1 UINT )  */



/*
    1.2  Tag numbers and counts
    ---------------------------
*/

#define ADC_LOSSCNT              20    /* LOSSCNT tag number                 */
#define ADC_SAMPLE               31    /* SAMPLE  tag number                 */

#define NBTAG_ADC_DEV            21    /* Number of tags for devices.        */
#define NBTAG_ADC_SUB            11    /* Number of sub-channel tags         */
#define NBTAG_ADC  (NBTAG_ADC_DEV \
                    +NBTAG_ADC_SUB)    /* Total Number of tags               */


/*
   1.3  Flags for supported bit depth per sample
   ---------------------------------------------
*/


#define ADC_4B                    0    /* Device support 4 bit samples.      */
#define ADC_6B                    2    /* Device support 6 bit samples.      */
#define ADC_8B                    4    /* Device support 8 bit samples.      */
#define ADC_10B                   6    /* Device support 8 bit samples.      */
#define ADC_12B                   8    /* Device support 12 bit samples.     */
#define ADC_16B                  10    /* Device support 16 bit samples.     */
#define ADC_20B                  12    /* Device support 20 bit samples.     */
#define ADC_24B                  14    /* Device support 24 bit samples.     */
#define ADC_28B                  16    /* Device support 28 bit samples.     */
#define ADC_32B                  18    /* Device support 32 bit samples.     */

/*
   1.4  IT_RX events error codes
   -----------------------------
*/

#define OK                        0    /* No error                           */
#define LOSS                      1    /* Data loss                          */
#define OVERRUN                   2    /* FIFO or DMA overrun error          */
#define ABORTRCV                  3    /* Abort receive                      */

/*
    2  Structures and typedefs
    __________________________

    2.1  High level ADC driver definitions - Adc_drvdef
    ---------------------------------------------------
*/


typedef struct                         /* High level ADC input parameters   */
  BEGIN                                /* ---------------------------------- */
    const CHAR    *uname             ; /* MUX input name                     */
    CHAR           fastcap           ; /* Fast sampling capacity 0:NO 1:YES  */
    CHAR           gaincap           ; /* Amplifier available 0:NO 1:YES     */
    CHAR           swpos             ; /* MUX input number/selector          */
    CHAR           lsfifonum         ; /* FIFO low-level number/identifier   */
    CHAR           lsdmanum          ; /* DMA stream low-level number/idenT  */
  END              Adc_inpdef        ; /* ---------------------------------- */

typedef struct                         /* High level ADC parameters          */
  BEGIN                                /* ---------------------------------- */
    IWORD          ldevnum           ; /* Low-level driver index             */
    UINT           vref              ; /* Vref value in milli-volts          */
    UINT           supp_depth        ; /* Sample depth supported by ADC      */
    UINT           supp_dma          ; /* Sample depth supported thru DMA    */
    UINT           list_sizes        ; /* Output sample sizes in bytes       */
    UINT           minfreq           ; /* ADC min sampling frequency in Hz   */
    UINT           maxfreq           ; /* ADC max sampling frequency in Hz   */
    UCHAR          maxfast           ; /* Maximum counts of fast samplings   */
    const Adc_inpdef  *inpdef        ; /* Inputs parameters table address    */
  END              Adc_drvdef        ; /* ---------------------------------- */

/*
    2.2  Low level ADC driver interface - Adc_desc
    ----------------------------------------------
*/
typedef struct                         /* Low-level procedure's addresses    */
  BEGIN                                /* ---------------------------------- */
    void           (*info       )()  ; /* Get start-up information on device */
    void           (*setpow     )()  ; /* Set power ON/OFF command           */
    void           (*setpar     )()  ; /* Configure ADC parameters           */
    void           (*setinp     )()  ; /* Select one MUX input               */
    int            (*getsmp     )()  ; /* Get a new sample                   */
                                       /* MODE=FASTSAMP, no DMA ------------ */
    void           (*startfifo  )()  ; /* Enable FIFO interrupts             */
    int            (*getfifosmp )()  ; /* Get a new sample from sample FIFO  */
    void           (*irqfifo    )()  ; /* Mask/Unmask the FIFO IRQ           */
    void           (*stopfifo   )()  ; /* Disable FIFO interrupt             */
                                       /* MODE=FASTSAMP, with DMA ---------- */
    void           (*infodma    )()  ; /* Get circular buffer align & size   */
    void           (*bufdma     )()  ; /* Give the circular buffer address   */
    void           (*startdma   )()  ; /* Start data transfer                */
    int            (*getdmasmp  )()  ; /* Extract received data              */
    void           (*irqdma     )()  ; /* Mask/Unmask the DMA IRQ            */
    void           (*stopdma    )()  ; /* Stop data transfer                 */
  END              Adc_desc          ; /* ---------------------------------- */

/*
    2.3  ADC input context - Adc_inpctx
    -----------------------------------
*/

typedef struct                         /* ADC MUX input context              */
  BEGIN                                /* ---------------------------------- */
    Req_subctx     rsctx             ; /* Sub-channel context   req_lib.c    */
    Adc_inpdef    *def               ; /* Input high level parameters        */
    UCHAR          dev               ; /* High level controller number       */
    UCHAR          inext             ; /* Next input context to be polled    */
                                       /* ---------------------------------- */
    UCHAR          mode              ; /* ONDEMAND SLOWSCAN FASTSAMP         */
    UCHAR          ihyst             ; /* IND_REPORT Hysteresis              */
    UCHAR          gain              ; /*                                    */
                                       /*                                    */
    UCHAR          sign              ; /* 0:UNSIGNED  1:SIGNED               */
    UCHAR          scaling           ; /* 0:RAW  1:NORMALIZED  2:SCALED      */
    UINT           scamul            ; /* SCALING=SCALED  Multiplier         */
    UINT           scadiv            ; /* SCALING=SCALED  Divider            */
    UINT           offhalf           ; /*                                    */
    UCHAR          norshift          ; /* Left shift for SCALING=NORMALIZED  */
                                       /*                                    */
    UINT           psamp             ; /* Last sampled value                 */

  END              Adc_inpctx        ; /* ---------------------------------- */

/*
    2.4  ADC device context - Adc_devctx
    ------------------------------------
*/

typedef struct                         /* ADC device context                 */
  BEGIN                                /* ---------------------------------- */
    Req_ctx        rctx              ; /* Allocation context.   req_lib.c    */
    Key_lnk        key               ; /* Intern job queues.    req_lib.c    */
                                       /* ---------------------------------- */
    Adc_drvdef     *def              ; /* Device generic definitions.        */
    Adc_desc       *desc             ; /* Low-level procedure adresses       */
    void           (*setinp    )()   ; /* Select one MUX input               */
    IWORD          (*getsmp    )()   ; /* Read a new sample from ADC         */
    void           (*start     )()   ; /* Stopŝ FIFO or DMA procedure addr   */
    IWORD          (*get       )()   ; /* Read a new sample from FIFO/DMA    */
    void           (*irq       )()   ; /* Disable/Enable FIFO or DMA IRQ     */
    void           (*stop      )()   ; /* Stopŝ FIFO or DMA procedure addr   */
                                       /* ---------------------------------- */
    const CHAR     *name             ; /* Hardware name string address       */
    UCHAR          depth             ; /* Output sample depth as enum        */
    UINT           scanfreq          ; /* Scanning frequency in Hz           */
    UINT           sampfreq          ; /* Sampling frequency in Hz           */
    UCHAR          interlv           ; /* Samples interleaving 0:NO 1:YES    */
    UCHAR          reqsize           ; /* Requested size DEFAULT 1 2 3 4     */
    UCHAR          reqend            ; /* Output samples 0:CPU  1:MSB  2:LSB */
    UCHAR          outsize           ; /* Output samples size in bytes 1..4  */
    UCHAR          outend            ; /* Output samples endianess           */
    UCHAR          outdma            ; /* 0:DMA not used  1:DMA used         */
    UCHAR          adcbits           ; /* Output sample depth in bits        */
    UINT           losscnt           ; /* Count of receive data losses       */
                                       /* ---------------------------------- */
    Adc_inpctx     *tinpctx          ; /* Address of MUX input context table */
    UCHAR          nbinps            ; /* Number of MUX input contexts       */
    UCHAR          nballoc           ; /* Count of allocated MUX inputs      */
    USHORT         fvl               ; /* First sub-channel logical way      */
    UCHAR          sfirst            ; /* First input context to scan        */
                                       /* ---------------------------------- */
    UINT           sampmsk           ; /* Mask for currently sampled inputs  */
    UCHAR          sampnb            ; /* Count of currently sampled inputs  */
    UCHAR          oorder[4]         ; /* Output samples byte order LUT      */
    INT            rcvif             ; /* User buffer interleave distance    */
    INT            ircvmax           ; /* Max value for "ircv", plus 1       */
    INT            bufif             ; /* Temp buffer interleave distance    */
    INT            ibufmax           ; /* Max value for "ibuf", plus 1       */
    INT            iadd              ; /* Increment for "ircv" and "ibuf"    */
                                       /* ---------------------------------- */
    UCHAR          buf[64]           ; /* Temporary buffer, if no user buf   */
    USHORT         ibuf              ; /* Index of the next byte to write    */
                                       /* ---------------------------------- */
    UCHAR         *rcv               ; /* Addess of the reception buffer     */
    USHORT         rcvsize           ; /* Size of the reception buffer       */
    USHORT         ircv              ; /* Index of the next byte to write    */
                                       /* ---------------------------------- */
    UCHAR          started           ; /* FIFO/DMA started                   */
    UINT           idto              ; /* SLOWSCAN timer identifier          */
  END              Adc_devctx        ; /* ---------------------------------- */

typedef struct                         /* ADC registers save                 */
  BEGIN                                /* ---------------------------------- */
    UCHAR          depth             ; /* Output sample depth as enum        */
    UINT           scanfreq          ; /* Scanning frequency in Hz           */
    UINT           sampfreq          ; /* Sampling frequency in Hz           */
    UCHAR          interlv           ; /* Samples interleaving 0:NO 1:YES    */
    UCHAR          reqsize           ; /* Requested size DEFAULT 1 2 3 4     */
    UCHAR          reqend            ; /* Output samples 0:CPU  1:MSB  2:LSB */
                                       /* ---------------------------------- */
    UCHAR          mode              ; /* ONDEMAND SLOWSCAN FASTSAMP         */
    UCHAR          ihyst             ; /* IND_REPORT Hysteresis              */
    UCHAR          gain              ; /*                                    */
    UCHAR          sign              ; /* 0:UNSIGNED  1:SIGNED               */
    UCHAR          scaling           ; /* 0:RAW  1:NORMALIZED  2:SCALED      */
    UINT           scamul            ; /* SCALING=SCALED  Multiplier         */
    UINT           scadiv            ; /* SCALING=SCALED  Divider            */
                                       /* Low-level setpar parameters        */
    UINT           adcbits           ; /* Selected ADC depth in bits         */
    UCHAR          outdma            ; /* 0:Use FIFO 1:Use DMA               */
    UINT           sampmsk           ; /* Mask for sampled inputs            */
  END              Adc_savreg        ; /* ---------------------------------- */


/*
    3  Public variables
    ___________________
*/

extern UCHAR       adc_numdrv        ; /* Driver identifier for board access.*/


/*
    4  Prototypes of public procedures
    __________________________________
*/

  void  itadcsmp            (IWORD                                           );


#endif                                 /* End of DRV_ADC_H.                  */

/*
    5  RAM data to be included in board_xyz.c
    _________________________________________

*/

#ifdef BOARD_XYZ_DATA_C

#define ADC_NBVL (1+ADC_NBDEV+ADC_NBINP)

const USHORT adc_nbiod  = ADC_NBIOD  ; /* Controllers seen by the VMOS       */
const USHORT adc_nbvl   = ADC_NBVL   ; /* Number of LW                       */
const USHORT adc_nbdev  = ADC_NBDEV  ; /* Number of device contexts          */
const USHORT adc_nbinp  = ADC_NBINP  ; /* Number of pin context's            */

USHORT       adc_states[ADC_NBVL]    ; /* Logical Ways states.               */
Adc_devctx   adc_devctx[ADC_NBDEV]   ; /* Controller's context table.        */
Adc_inpctx   adc_inpctx[ADC_NBINP]   ; /* Pins' contexts table.              */

#endif                                 /* End of BOARD_XYZ_C                 */


