/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_dma.h"             */
/*  Constants and definitions for DMA driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 10/27/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Values for the "code" field of Dma_job
            1.2  Values for the "opt" field of Dma_job
            1.3  Values for the "type" field of S_mem
            1.4  Values for the "acc" parameter of begacc/endacc
            1.5  Values for "type" field of Dma_desc
            1.6  Values for "type" field of Dma_drvdef
            1.7  Values for the "" field of Dma_ctx

         2  Structures and typedefs
            2.1  Board structure for low-level procedure addresses
            2.2  Board structure for high level driver parameters
            2.3  Source or destination parameter
            2.4  One DMA request description
            2.5  One DMA channel context
            2.6  Set of interrupt handler call-bak

         3  Public variables

         4  Prototypes of public procedures


*/


#ifndef DRV_DMA_H                      /* If this file is not already        */
#define DRV_DMA_H                      /* included, include it.              */

/*
    1    Defines
    ____________
*/

/* 1.1  Values for the "code" field od Dma_job ..............................*/

#define J_CJUMP              0         /* Conditionnal jump                  */
#define J_TRIG               1         /* Send a trigger                     */
#define J_WAIT               2         /* Wait for a trigger                 */

#define J_MEMSND             3         /* copy from RAM to register          */
#define J_MEMRCV             4         /* copy from register to RAM          */
#define J_MEMCPY             5         /* use the memcpy function            */
#define J_DMASND             6         /* use the "dma_syx_snd" function     */
#define J_DMARCV             7         /* use the "dma_syx_rcv" function     */
#define J_DMACPY             8         /* use the "dma_syx_cpy" function     */
#define J_CIRSND             9         /* copy from RAM to hardware circular */
#define J_CIRRCV             10        /* copy from hardware circular to RAM */

#define J_ALLOC              11        /* Allocates one frame buffer         */
#define J_FREE               12        /* Frees one frame buffer             */
#define J_SLOTSND            13        /* Starts a DMA SLOT in sending mode  */
#define J_SLOTRCV            14        /* Starts a DMA SLOT in receiving mode*/
#define J_SLOTSETSND         15        /* Sets one slot frame buffer         */
#define J_SLOTSETRCV         16        /* Sets one slot frame buffer         */

#define J_LAST        J_SLOTSETRCV     /* Last job code value                */

/* 1.2  Values for the "opt" field of Dma_job ...............................*/

#define O_SWAP2          0x0001        /* Two  bytes swap supported          */
#define O_SWAP4          0x0002        /* Four bytes swap supported          */
#define O_SNDSTRM        0x0004        /* Send a continuous data stream      */
#define O_RCVSTRM        0x0008        /* Receive a continuous data stream   */

#define O_SWAP (O_SWAP2 | O_SWAP4)
#define O_STRM (O_SNDSTRM | O_RCVSTRM)

#define O_OPTBIT (O_SWAP | O_STRM)

/* 1.3  Values for the "type" field of S_mem ................................*/

#define REG1                  1        /* Area is a 1 byte size register     */
#define REG2                  2        /* Area is a 2 byte size register     */
#define REG4                  4        /* Area is a 4 byte size register     */
#define SEG                   5        /* Area is a RAM segment              */
#define CIR                   6        /* Area is a RAM circular buffer      */

/* 1.4  Values for the "acc" parameter of begacc/endacc .....................*/

#define A_READ                0        /* Circular buffer read access        */
#define A_WRITE               1        /* Circular buffer write access       */

/* 1.5  Values for "type" field of Dma_desc .................................*/

#define T_DMA_DESC            (1<<0)   /* Standard DMA functions available   */
#define T_SLOT_DESC           (1<<1)   /* SLOT DMA functions available       */

/* 1.6  Values for "type" field of Dma_drvdef ...............................*/

#define T_DMA_GP              0        /* Type for General purpose DMA       */
#define T_DMA_SP              1        /* Type for Special purpose DMA       */
#define T_DMA_SLOT            2        /* Type for DMA SLOT device           */

/* 1.7  Values for the "state" field of Dma_ctx .............................*/

#define IDLE                  0        /* No DMA transfer / No interrupt     */
#define ONERCV                1        /* One shot receive                   */
#define ONESND                2        /* One shot send                      */
#define ONECPY                3        /* One shot copy                      */
#define CIRCRCV               4        /* Continuous/circular receive        */
#define CIRCSND               5        /* Continuous/circular sending        */
#define SLOTRCV               6        /* DMA SLOT receive                   */
#define SLOTSND               7        /* DMA SLOT sending                   */

/* 1.8  Values for the "evtwait" field of Dma_ctx ...........................*/

#define T_RUN                 0        /* No J_TRIGWAIT                      */
#define T_WAIT                1        /* Blocked by J_TRIGWAIT              */
#define T_READY               2        /* In the "ready to run " queue       */

/* 1.9  Values for the "evtnum" field of Dma_ctx ............................*/

#define E_TRIG                0        /* Received event = trigger           */
#define E_SND                 1        /* Received event = end of sending    */
#define E_RCV                 2        /* Received event = end of receive    */
#define E_CPY                 3        /* Received event = end of copy       */

/* 1.10 Values for the "evtmask" field of Dma_ctx ...........................*/

#define W_TRIG     (1 << E_TRIG)       /* Waiting for a trigger O_TRIG       */
#define W_SND      (1 << E_SND )       /* Waiting for a SND end IRQ          */
#define W_RCV      (1 << E_RCV )       /* Waiting for a RCV end IRQ          */
#define W_CPY      (1 << E_CPY )       /* Waiting for a CPY end IRQ          */

/*
    2    Structures and typedefs
    ____________________________
*/

/* 2.1  Board structure for low-level procedure addresses ...................

*/

typedef struct                         /* DMA Low level procedures           */
  BEGIN                                /* ---------------------------------- */
    UCHAR             type           ; /* T_DMA_DESC + T_SLOT_DESC           */
    IWORD             (*info)    ()  ; /* Name of the DMA Device             */
    IWORD             (*init)    ()  ; /* Initializes the DMA controller     */
    IWORD             (*end)     ()  ; /* End of use of the DMA controller   */
    IWORD             (*init_hch)()  ; /* Initializes one hardware channel   */
    IWORD             (*end_hch) ()  ; /* End of use of one hardware channel */
    IWORD             (*abort)   ()  ; /* Aborts an on-going transfer        */
    IWORD             (*irq)     ()  ; /* Disable/Enable the DMA IRQ         */
    IWORD             (*setitcb) ()  ; /* Declare/Undeclare user IRQ CB      */
                                       /* ---------------------------------- */
    IWORD             (*snd)     ()  ; /* RAM ro register transfer start     */
    IWORD             (*rcv)     ()  ; /* Register to RAM transfer start     */
    IWORD             (*cpy)     ()  ; /* RAM to RAM transfer start          */
    IWORD             (*begacc)  ()  ; /* Begin circular buffer access       */
    IWORD             (*endacc)  ()  ; /* End   circular buffer access       */
                                       /* ---------------------------------- */
    IWORD             (*snds)    ()  ; /* Slot to register transfer start    */
    IWORD             (*rcvs)    ()  ; /* Register to Slot transfer start    */
    IWORD             (*set)     ()  ; /* Sets a slot buffer                 */
    IWORD             (*get)     ()  ; /* Gets a slot buffer                 */
  END                 Dma_desc       ; /* ---------------------------------- */


/* 2.2  Board structure for high level driver parameters ....................*/

typedef struct                         /* DMA High level driver params       */
  BEGIN                                /* ---------------------------------- */
    UINT              ldevnum        ; /* Device index for low level driver  */
    UCHAR             nbchan         ; /* Number of channels                 */
    UCHAR             type           ; /* 0:General Purpose 1:Specialized    */
    const UCHAR       *maxuser       ; /* Maximum numbers of allowed alloc's */
    USHORT            opt            ; /* Available options bitmaps          */
    USHORT            thr            ; /* Memcpy/DMA usage limit             */
  END                 Dma_drvdef     ; /* ---------------------------------- */


/* 2.3  Source or destination parameter .....................................*/

typedef struct                         /* Source or destination DMA area     */
  BEGIN                                /* ---------------------------------- */
    UCHAR             *adr           ; /* Area base address                  */
    UINT              size           ; /* Area size in bytes                 */
    UCHAR             type           ; /* Area description bits              */
                                       /* ---------------------------------- */
    UINT              inoff          ; /* Next byte to write offset          */
    UINT              incnt          ; /* Bytes beeing inputted              */
    UINT              outoff         ; /* Next byte to read offset           */
    UINT              outcnt         ; /* Bytes beeing outputted             */
    UINT              cnt            ; /* Byte count                         */
  END                 S_mem          ; /* ---------------------------------- */


/* 2.4  One DMA request description .........................................*/

typedef struct        dma_job          /* One DMA request                    */
  BEGIN                                /* ---------------------------------- */
    UCHAR             code           ; /* Code of the requested operation    */
    UCHAR             *src           ; /* Transfer data source               */
    UCHAR             *dst           ; /* Transfer data destination          */
    UCHAR             srctype        ; /* Source description type            */
    UCHAR             dsttype        ; /* Desctination description type      */
    INT               cnt            ; /* Number of bytes to be copied       */
    UINT              par1           ; /* Parameter                          */
    UINT              par2           ; /* Parameter                          */
    USHORT            opt            ; /* Option bits for transfer           */
                                       /* ---------------------------------- */
    IWORD             (*cb)()        ; /* End of transfer call-back addr     */
    void              *cbadr         ; /* First parameter to be given to cb  */
    USHORT            userid         ; /* Channel identifier                 */
    USHORT            reqid          ; /* 2nd   parameter to be given to cb  */
    S_mem            *srcmem         ; /* Source area structure address      */
    S_mem            *dstmem         ; /* Destination area structure address */
                                       /* ---------------------------------- */
    struct dma_job   *next           ; /* Link addr to next job              */
    UCHAR             chanidx        ; /* Index in "dma_ctx[]"               */
  END                 Dma_job        ; /* ---------------------------------- */
 

/* 2.5  One DMA channel context .............................................*/

#define SZSLOT        4                /* bufadr[]/bufsz[] array sizes       */

typedef struct                         /* One DMA channel context            */
  BEGIN                                /* ---------------------------------- */
    const Dma_desc   *desc           ; /* Low level procedures               */
    const Dma_drvdef *def            ; /* High level driver parameters       */
    UCHAR             dev            ; /* DMA controller high level number   */
    UCHAR             chanidx        ; /* This structure index in "dma_ctx"  */
    UCHAR             ldevnum        ; /* DMA controller low level number    */
    UCHAR             hchan          ; /* DMA channel    low level  number   */
                                       /* ---------------------------------- */
    UCHAR             usernb         ; /* 0:free >0:allocated                */
    UINT              userbtm        ; /* Allocation bitmap                  */
    UCHAR             state          ; /* IDLE/ONERCV/ONESND/ONECPY/CIRCRCV  */
    UCHAR             masked         ; /* 0:Unlocked 1:Locked                */
    UCHAR             more           ; /* More room/data available (hard cir)*/
    UCHAR             link           ; /* Link for "trigwait=T_READY"        */
                                       /* Execution flow control ----------- */
    Dma_job          *job            ; /* Linked list of jobs head           */
    Dma_job          *jobptr         ; /* Current executed job               */
    Dma_job          *jobnext        ; /* Next executed job                  */
    Dma_job          *jobrcv         ; /* RCV job waiting or an IRQ          */
    Dma_job          *jobsnd         ; /* SND job waiting or an IRQ          */
    Dma_job          *jobcpy         ; /* CPY job waiting or an IRQ          */
    UINT              off            ; /* Count of transferred bytes         */
                                       /* Only for CIRCRCV/CIRCSND --------- */
    UCHAR*            lastregadr     ; /* Last register address              */
    INT               lastregsz      ; /* Last register type/size            */
                                       /* Only for J_TRIG/J_WAI ......------ */
    UCHAR             evtwait        ; /* T_RUN/T_WAIT/T_READY               */
    UCHAR             evtmask        ; /* Bitmap of W_TRIG/W_SND/W_RCV/W_CPY */
    UCHAR             evtnum         ; /* E_TRIG/R_SND/E_RCV/E_CPY           */
    UCHAR             evtcnt[4]      ; /* Counter of trig/snd/rcv/cpy        */
    UCHAR            *evtbuf[4]      ;
    USHORT            evtlen[4]      ;
                                       /* Only for J_ALLOC/J_SETSLOT ------- */
    UCHAR            *bufadr[SZSLOT] ; /* Last allocated buffer address      */
    USHORT            bufsz [SZSLOT] ; /* Last allocated buffer size         */
    UCHAR             nbslot         ; /* Current slot count value           */
    UCHAR             fstidx         ; /* Read index in "bufadr()/bufsz[]"   */
    UCHAR             mididx         ; /* Next to be given to low level      */
    UCHAR             lastidx        ; /* Write index in "bufadr[]/bufsz[]"  */
    UCHAR             nbused         ; /* Count of used entries              */
                                       /*                                    */
    UCHAR             maxslotlow     ; /* Max low slots number               */
    UCHAR             nbslotlow      ; /* Used low slots number              */
    UCHAR             fstidxlow      ; /* Next low slot index                */
    UCHAR             lastidxlow     ; /* Next low slot index                */
    UCHAR             nbusedlow      ; /* Count of used entries              */
                                     ; /* Errors --------------------------- */
    void              (*errcb)()     ; /* Error user call-back address       */
    void              *errcbadr      ; /* First parameter for "errcb"        */
    UINT              errsndcnt      ; /* Counter of sending underrun        */
    UINT              errrcvcnt      ; /* Counter of receive overrun         */
  END                 Dma_ctx        ; /* ---------------------------------- */

/* 2.6  Set of interrupt handler call-back ..................................*/

typedef struct                         /* One set of IRQ handler call-backs  */
  BEGIN                                /* ---------------------------------- */
    void              (*dmasnd)()    ; /* End of sending IRQ call-back       */
    void              (*dmarcv)()    ; /* End of receiving IRQ call-back     */
    void              (*dmacpy)()    ; /* End of RAM 2 RAM copy IRQ call-back*/
    void              (*circsnd)()   ; /* End of circular sending IRQ cb.    */
    void              (*circrcv)()   ; /* End of circular receiving IRQ cb.  */
    void              (*slotsnd)()   ; /* End of slot sending IRQ call-back  */
    void              (*slotrcv)()   ; /* End of slot receiving IRQ call-back*/
  END                 Dma_itcb       ; /* ---------------------------------- */

typedef struct                         /* One set of IRQ handler call-backs  */
  BEGIN                                /* ---------------------------------- */
    void              (*proc[5])()   ; /*                                    */
  END                 Dma_itproc     ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/



/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD  dma_init            (INT                                              );
IWORD  dma_end             (INT                                              );
IWORD  dma_alloc_gpchan    (UINT,INT,INT*                                    );
IWORD  dma_alloc_spchan    (INT,INT,INT,INT*                                 );
IWORD  dma_free_chan       (INT                                              );
IWORD  dma_start           (INT,S_mem*,S_mem*,INT,INT,UINT,UINT,
                            IWORD(*)(),void*,INT                             );
IWORD  dma_tasklet         (INT,Dma_job*,INT,INT                             );
IWORD  dma_abort           (INT,INT                                          );
IWORD  dma_set_errcb       (INT,void(*)(),void*                              );
IWORD  dma_cb_evt          (S_evt*,INT,INT,INT                               );

IWORD    smem_init_register(S_mem*,UCHAR*,UINT                               );
IWORD    smem_init_segment (S_mem*,UCHAR*,UINT                               );
IWORD    smem_init_circular(S_mem*,UCHAR*,UINT                               );

IWORD  smem_init           (S_mem*,UCHAR*,UINT,INT                           );
void   smem_reset          (S_mem*                                           );
void   smem_begin_input    (S_mem*,UINT                                      );
void   smem_end_input      (S_mem*,UINT                                      );
void   smem_begin_output   (S_mem*,UINT                                      );
void   smem_end_output     (S_mem*,UINT                                      );
UCHAR* smem_rdadr          (S_mem*                                           );
UCHAR* smem_wradr          (S_mem*                                           );
UINT   smem_free           (S_mem*                                           );
UINT   smem_avail          (S_mem*                                           );

void   itdmasnd            (INT,INT,INT                                      );
void   itdmarcv            (INT,INT,INT                                      );
void   itdmacpy            (INT,INT,INT                                      );
void   itslotsnd           (INT                                              );
void   itslotrcv           (INT                                              );

void   fn_loop             (Dma_job*                                         );
void   fn_evtnum           (Dma_job*                                         );

#endif                                 /* End of DRV_DMA_H.                  */

#ifdef BOARD_XYZ_DATA_C

const UCHAR dma_max_ctx = DMA_MAX_CTX;
const UCHAR dma_max_job = DMA_MAX_JOB;
Dma_ctx     dma_ctx[DMA_MAX_CTX]     ;
Dma_job     dma_job[DMA_MAX_JOB]     ;

#endif                                 /* End of BOARD_XYZ_C                 */


