/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_eep.h"             */
/*  Constants and definitions for I2C driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 12/19/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  User Data types codes
            1.2  Interface procedure identifiers (event "flags" values)

         2  Structures and typedefs
            2.1  I2C EEPROM context - I2c_eepctx
            2.2  EEPROM low level - Eep_devdesc / Eep_drvdef
            2.3  EEPROM context - Eep_ctx

         3  Public variables
         4  Prototypes of public procedures

         5  RAM data to be inclueded in board_xyz.c
*/


#ifndef DRV_EEP_H                      /* If this file is not already        */
#define DRV_EEP_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  User Data types codes
    --------------------------
*/

#define TYP_MEM            0           /* eep_writem, eep_readmem            */
#define TYP_STR            1           /* eep_writestr, eep_readstr          */
#define TYP_INT            2           /* eep_writeint, eep_readint          */

/*
    1.2  Interface procedure identifiers (event "flags" values)
    -----------------------------------------------------------
*/

#define WINIT     ((0<<8) | 0xFF   )   /* eep_waitinit                       */
#define WRMEM     ((1<<8) | TYP_MEM)   /* eep_writemem                       */
#define WRSTR     ((2<<8) | TYP_STR)   /* eep_writestr                       */
#define WRINT     ((3<<8) | TYP_INT)   /* eep_writeint                       */
#define RDMEM     ((4<<8) | TYP_MEM)   /* eep_readmem                        */
#define RDSTR     ((5<<8) | TYP_STR)   /* eep_readstr                        */
#define RDINT     ((6<<8) | TYP_INT)   /* eep_readint                        */



/*
    2    Structures and typedefs
    ____________________________


    2.1  I2C EEPROM context - I2c_eepctx
    ------------------------------------
*/


typedef struct                         /* Contexte I2c pour une EEPROM       */
  BEGIN                                /* ---------------------------------- */
    UCHAR         *cacheadr          ; /* Adresse du cache de l'EEPROM       */
    INT            rowsize           ; /* Taille en octets d'une rangee      */
  END              I2c_eepctx        ; /* ---------------------------------- */
 

/*
    2.2  EEPROM low level - Eep_devdesc / Eep_drvdef
    ------------------------------------------------
*/


typedef struct                         /* EEPROM low-level abstraction       */
  BEGIN                                /* ---------------------------------- */
    CHAR         *name               ; /* Low-level driver name              */
    IWORD         (*init)    ()      ; /* Initialization                     */
    IWORD         (*send)    ()      ; /* Write to EEPROM                    */
    IWORD         (*receive) ()      ; /* Read from EEPROM                   */
  END             Eep_devdesc        ; /* ---------------------------------- */

typedef struct                         /* EEPROM description parameters      */
  BEGIN                                /* ---------------------------------- */
    CHAR         *name               ; /* EEPROM device name                 */
    UCHAR         bus                ; /* EEPROM bus number                  */
    UCHAR         wadr               ; /* EEPROM I2C write address (even)    */
    UCHAR        *cacheadr           ; /* EEPROM RAM cache address           */
    UCHAR         eepsubsz           ; /* EEPROM address/offset size in bits */
    UCHAR         rowsize            ; /* Row size in bytes                  */
  END             Eep_drvdef         ; /* ---------------------------------- */


/*
    2.3  EEPROM context - Eep_ctx
    -----------------------------
*/

typedef struct                         /* One waiting job queue entry        */
  BEGIN                                /* ---------------------------------- */
    USHORT        code               ; /* Event code REQ_WRITE or REQ_READ   */
    USHORT        aut                ; /* Requester FSM number               */
    USHORT        vl                 ; /* Requester LW  number               */
    UCHAR*        buf                ; /* Data buffer address                */
    USHORT        lg                 ; /* Count of bytes to be read/written  */
    USHORT        flags              ; /* Interface proc number and user type*/
    INT           eepadr             ; /* EEPROM offset                      */
    USHORT        reqid              ; /* User request identifier            */
  END             Eep_job            ; /* ---------------------------------- */

#define MAX_JOBE   8

typedef struct                         /* AUT_EEP context                    */
  BEGIN                                /* ---------------------------------- */
    INT           iod                ; /* VMOS EEPROM IOD                    */
    IWORD         (*init)     ()     ; /* Low-level "init"    function       */
    IWORD         (*send)     ()     ; /* Low level "send"    function       */
    IWORD         (*receive)  ()     ; /* Low level "receive" function       */
                                       /*                                    */
    Eep_job       job[MAX_JOBE]      ; /* EEP jobs wait circular queue       */
    UCHAR         lec                ; /* Queue read index                   */
    UCHAR         ecr                ; /* Queue write index                  */
    UCHAR         nbjob              ; /* Count of jobs in the queue         */
                                       /*                                    */
    Eep_drvdef   *eep                ; /* Parameters structure address       */
  END             Eep_ctx            ; /* ---------------------------------- */
                                                                             
typedef struct                         /* EEP interface procedure context    */
  BEGIN                                /* ---------------------------------- */
    Eep_drvdef   *eep                ; /* Parameters structure address       */
    Eep_devdesc  *eeplow             ; /* Low level driver structure addr    */
  END             Eepproc_ctx        ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Eep_devdesc i2c_eep_devdesc ;


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD eep_get_nb          (INT*                                            );
  IWORD eep_get_info        (INT,INT*,UCHAR**,CHAR*,CHAR*,INT*               );
  IWORD eep_wait_init       (INT,INT,INT*                                    );
  IWORD eep_writemem        (INT,INT,UCHAR*,INT,INT,INT*                     );
  IWORD eep_writestr        (INT,INT,CHAR*,INT,INT,INT*                      );
  IWORD eep_writeint        (INT,INT,INT,INT,INT,INT*                        );
  IWORD eep_writeint1       (INT,INT,INT,INT*                                );
  IWORD eep_writeint2       (INT,INT,INT,INT*                                );
  IWORD eep_writeint4       (INT,INT,INT,INT*                                );
  IWORD eep_readmem         (INT,INT,UCHAR*,INT,INT,INT*                     );
  IWORD eep_readstr         (INT,INT,CHAR*,INT,INT,INT*                      );
  IWORD eep_readint         (INT,INT,INT*,INT,INT,INT*                       );
  IWORD eep_readint1        (INT,INT,INT*,INT*                               );
  IWORD eep_readint2        (INT,INT,INT*,INT*                               );
  IWORD eep_readint4        (INT,INT,INT*,INT*                               );


#endif                                 /* End of DRV_EEP_H.                  */

/*
    5  RAM data to be inclueded in board_xyz.c
    ___________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

const UCHAR   eep_ctxmax = NUM_EEP   ; /* Count of EEPROMs                   */
Eep_ctx       eep_ctx    [NUM_EEP]   ; /* EEP context table                  */
USHORT        eep_etat   [NUM_EEP]   ; /* LW state variables                 */
Eepproc_ctx   eepproc_ctx[NUM_EEP]   ; /* Interface functions context table  */


#endif                                 /* End of BOARD_XYZ_C                 */

