/*
**  Myio.c - Sample code for simple I/O functions ============== 09/11/19 =  **
**                                                                           **
**  This file is a library of subroutines that are used by "myapp.c" in      **
**  order to do I/O's. They provide a simplified interface compared to the   **
**  "full" I/O interface of "drv.c". The provided subroutines are:           **
**                                                                           **
**  int myio_open(int driver,int device,int *iod)                            **
**  int myio_close(int iod)                                                  **
**  int myio_alloc_sub(int iod, const char *tagini, int *iodsub)             **
**  int myio_free_sub(int iodsub)                                            **
**  int myio_getval(int iod, char *taglist)                                  **
**  int myio_setval(int iod, const char *taglist)                            **
**  int myio_givetok(int iod, int nbtok, int size, int *count)               **
**  int myio_write(int iod, unsigned char *buf, int lg)                      **
**  int myio_send(int iod, unsigned char *buf, int lg, int *count)           **
**                                                                           **
**  Currently those procedures are supporting 5 drivers that are the         **
**  asynchronous driver "ASY", the pin's driver "GPIO", the led driver       **
**  "LED", the keyboard interface "KBDITF" and the DAC driver "DAC"          **
**                                                                           **
**  The protopypes of the subroutines are in "myio.h" that must be included  **
**  in "myapp.c". This include file also contains defines that are           **
**  intended to be used as the first parameter of "myio_open" (DRVASY,       **
**  DRVGPIO, DRVLED, DRVKBD and DRV_DAC) and also defines that are the       **
**  "myio_open" second parameter possible values (DEV0, DEV1)                **
**                                                                           **
**  The "myio_open" procedure opens in non-blocking mode the two ASY and     **
**  DAC drivers that supports writes (ASY and DAC) and reads (ASY). A non    **
**  blocking read is started using "myio_givetok", a non blocking write is   **
**  started with "myio_send". Also, despite the fact the driver is opened    **
**  in non-blocking IO mode, a blocking write may be done, using the         **
**  "myio_write" subroutine, that unschedule until the RESP_WRITE event is   **
**  received by the calling task.                                            **
**                                                                           **
**  The two GPIO and LED driver are using the concept of "sub-channel". With **
**  the GPIO driver we have one sub-channel per pin/GPIO and with the LED    **
**  driver we have one sub-channel per led. The two subroutines              **
**  "myio_alloc_sub" and "myio_free_sub" must be used, they respectively     **
**  enable to allocate one GPIO/LED (one subchannel) and to free one GPIO/   **
**  LED (so one sub-channel).                                                **
**                                                                           **
**  The "myio_setval" and "myio_getval" subroutines may be used with all     **
**  the 5 drivers.                                                           **
**                                                                           **
**  =======================================================================  **
*/

/* Includes files and external references ...................................*/

#include <stdtyp.l>
#include <vmk.h>                       /* vmk.c and vmio.c interface         */
#include <drv.h>                       /* drv.c interface                    */
#include <text.h>                      /* Prototype hsprintf().              */
#include <telecom.h>                   /* Constants and definitions for tags.*/
#include <engine.h>                    /* RESP_WRITE                         */
#include <systemm.txt>                 /* WAIT_CODERES                       */

#include "./myio.h"                    /* Procedure's prototypes             */


/* Static data of this module ---------------------------------------------- */

typedef struct                         // Parameters for drv_alloc_device
  {                                    // ----------------------------------
    char       *mname                ; // Module/driver name
    int        flags                 ; // Flags
  }            Allpar                ; // ----------------------------------

const Allpar allpar[MAX_MODD] =        // Those are the the names to be
  {                                    // given to "drv_init_driver" -------
    "ASY"    , NBLOCKING_IO          , // ASY  driver
    "GPIO"   , 0                     , // GPIO driver
    "LED"    , 0                     , // LED driver
    "KBDITF" , 0                     , // KBD driver
    "DAC"    , NBLOCKING_IO          , // DAC driver
  }                                  ; // ----------------------------------

const int rtepar[MAX_MODD][2] =        // Parameters for add_uroute
  {                                    // ----------------------------------
    IND_REPORT     , IND_REPORT      , // ASY  driver
    IND_REPORT     , IND_REPORT      , // GPIO driver
    -1             , -1              , // LED  driver
    DRV_KEY_PRESS  , DRV_KEY_RELEASE , // KBD  driver
    -1             , -1              , // DAC  driver
  }                                  ; // ----------------------------------


/* Internal global variables of this module -------------------------------- */

static int          modd [MAX_MODD]  ; // MODD of driver's
static int          rteid[MAX_MODD]  ; // User event route identifier
static char         iodnb[MAX_MODD]  ; // Count of allocated devices
static int          iodrv[MAX_MODD]    // For each driver, the list of IOD's
                         [MAX_DEV ]  ; // of opended devices

/* Prototypes of internal procedures --------------------------------------- */

static int chk_iod  (int,int*,int*                                          );

/*  Procedure myio_open -------------------------------------------------------

    Purpose : Initilize the driver if needed, then allocates and opens a device
*/

int myio_open(int driver,int device,const char* taglist,int *iod)
  {

    int             ret              ; // Returned code
    const Allpar    *par             ; // Address of drv_alloc parameters
    char            name[32]         ; // Device name to be opened
    int             major            ; // Driver major number << 16
    int             maxres , minres  ; // Values for add_uroute
    int             maxcode, mincode ; // Values for add_uroute

/*****************************************************************************
 * Step 1 : Check parameters values and exit with -1 if there is any error   *
 * ------   "driver" must be between 0 and MAX_MODD-1 and "device" must be   *
 *          between 0 and MAX_DEV-1                                          *
 *****************************************************************************/

    ret = -1                         ; // Let's assume it will fail
                                       
    if (driver < 0         ||          // The "driver" value must be between
        driver >= MAX_MODD ||          // 0 and MAX_MODD-1
        device < 0         ||          // The "device" value must be between
        device >= MAX_DEV    )         // 0 and MAX_DEV-1
      goto end                       ; // Otherwise we have an error

    par = allpar                       // Addres of the "allpar[driver]"
        + driver                     ; // parameter's structure


/*****************************************************************************
 * Step 2 : If the driver is not yet initialized, if the "modd[driver]"      *
 * ------   global variable value is now 0, then we have to send a REQ_INIT  *
 *          initialization request event to the VMIO driver FSM. To do this, *
 *          we call the "drv_init_driver" procedure. The VMIO driver FSM     *
 *          responds with a RESP_INIT event, we here unschedule until        *
 *          reception of this of this response event                         *
 *****************************************************************************/

    if( modd[driver] ) goto step4    ; // If driver is already initialized

    drv_init_driver(par->mname    ,    // Name of the driver module
                    0             ,    // Node number = local (0)
                    1             ,    // Reqid
                    &modd[driver] ,    // Module identifier MODD
                    &ret           ) ; // Return code

    if ( ret ) goto end              ; // Exit if error


/*****************************************************************************
 * Step 3 : If necessary, create one and one only user route per driver in   *
 * -------  order to receive the unsollicited events, that are in our case:  *
 *          - IND_REPORT for the ASY driver                                  *
 *          - IND_REPORT for the GPIO driver                                 *
 *          - DRV_KEY_PRESS and DRV_KEY_RELEASE for the KBD driver           *
 *          The DAC and the LED do not generate unsollicited events. We use  *
 *          the "rtepar[5][2]" consta table that contains the "mincode" and  *
 *          "maxcode" values to be given to "add_route". When no route has   *
 *          to be created, we put a -1 for the "mincode" value (LED and DAC) *
 *          - With the KBDITF, we may have two event codes that are          *
 *            DRV_KEY_PRESS (0) and DRV_KEY_RELEASE (1). The smallest one is *
 *            used as "mincode" (0) and the greatest one is "maxcode" (1)    *
 *            The "reserve" field of those events is the keycode and we      *
 *            accept any value, therefore we use "minres = maxres = -1"      *
 *          - With ASY and GPIO, events are IND_REPORT and we use in that    *
 *            case "mincode = maxcode = IND_REPORT". The "reserve" field of  *
 *            the IND_REPORT is the device IOD (ASY) or the sub-channel IOD  *
 *            (GPIO). We extract the "major number" from the MODD and we     *
 *            compute "minres" = "major" + 0x01000000 and "maxres" = "major" *
 *            + 0x0100FFFF. The "major" is bits b23..16 in both MODD and IOD *
 *            So we have a route that will "match" with all the ASY or LED   *
 *            events, whatever are the minor number and the sub-channel      *
 *          ---------------------------------------------------------------- *
 *          The structure of any MODD is:                                    *
 *          - Bits b31..28 are 0000  for an MODD or IOD                      *
 *          - Bits b27..24 are 1111  for a MODD                              *
 *          - Bits b23..16 are the major number                              *
 *          - Bits b15..0  are the selector (MODD index)                     *
 *          ---------------------------------------------------------------- *
 *          With IND_REPORTS events, the "reserve" fiels is the IOD. The     *
 *          structure of any IOD is as follows:                              *
 *          - Bits b31..28 are 0000  for an MODD or IOD                      *
 *          - Bits b27..24 are 0001  for a character driver                  *
 *          - Bits b23..16 are the major number                              *
 *          - Bits b15..0  are the minor number                              *
 *            For a character driver, this field has two sub-fields          *
 *            . Bits b15..8  are the device number                           *
 *            . Bits b7..0   are the sub-channel number, 0xFF for the device *
 *****************************************************************************/

    mincode = rtepar[driver][0]      ; // Extract the route "mincode"
    maxcode = rtepar[driver][1]      ; // Extract the route "maxcode"
    minres  = -1                     ; // With DRV_KEY_PRESS and
    maxres  = -1                     ; // DRV_KEY_RELEASE, we accept all the
                                       // values or "reserve" (it is keycode)
    if (mincode == -1) goto step4    ; // If no route has to be created then
                                       // skip the "add_uroute" call
    if (mincode == IND_REPORT)         // With IND-REPORT, "reserve" is the
      {                                // device or sub-channel IOD
        major  = modd[driver]          // Extract frol the MODD the major
               & 0x00FF0000          ; // number and keep it in b23..16
        minres = major | 0x01000000  ; // This interval will match all the
        maxres = major | 0x0100FFFF  ; // IOD's for the driver, all the 
      }                                // minor and sub-channels

    add_uroute(mincode    ,            // Min value for "code"
               maxcode    ,            // Max value for "code"
               minres     ,            // Min value for "reserve"
               maxres     ,            // Max value for "reserve"
               0          ,            // Flags
               &rteid[driver] )      ; // Returned Route identifier


/*****************************************************************************
 * Step 4 : Check that the device "device" is not currentl allocated and exit*
 * ------   with ret = -1 if this happens. For each device of each driver    *
 *          we keep the device IOD in "iodrc[driver][device]" for any        *
 *          allocated device or we have a 0 value when the device is free    *
 *          (Please note that an valid IOD is never a 0 value). So here we   *
 *          exit with -1 if "iodrv[driver][device]" is not 0.                *
 *****************************************************************************/

    step4 : ret = -1                 ; // Let's assume it will fail

    if ( iodrv[driver][device] )       // If the device has already been
      goto end                       ; // opened, exit with error -1


/*****************************************************************************
 * Step 5 : The VMIO driver is ready. We are going now to allocate the       *
 * ------   device. We call the "drv_alloc_device" procedure, this procedure *
 *          sends a REQ_ALLOC event to the VMIO driver FSM. This automaton   *
 *          will then send us a RESP_ALLOC response event. Here we           *
 *          unschedule until this event is received. We keep the returned    *
 *          IOD value in "iodrv[driver][devive]" and we increment the count  *
 *          "iodnb[driver]".                                                 *
 *****************************************************************************/

    hsprintf(name,"%s\\DEV%d",         // The device name is
             par->mname      ,         // "DRIVER_NAME\DEVn" with "n"
             device           )      ; // is the device number
    
    drv_alloc_device(modd[driver],     // Module Descriptor Id
                     name        ,     // Device name = name
                     ""          ,     // Tagini  = empty
                     ""          ,     // Taglist = empty
                     par->flags  ,     // Flags   = 0 or NBLOCKING_IO
                     2           ,     // Reqid
                     0           ,     // Route ID   = empty
                     0           ,     // Route ID 2 = empty
                     iod         ,     // Returned I/O descriptor
                     &ret         )  ; // Return code

    if ( ret ) goto end              ; // Exit if error

    iodrv[driver][device] = *iod     ; // Keep the IOD value
    iodnb[driver] ++                 ; // Update count of allocated devices


/*****************************************************************************
 * Step 6 : Here we initialize the hardware of the device. We call the       *
 * ------   "device_open_device" procedure. This procedures sends a REQ_OPEN *
 *          request event to the VMIO driver automaton, that is located      *
 *          inside the IO container. Then we ask to be unscheduled until the *
 *          AUT_LED send us the RESP_OPEN response event                     *
 *****************************************************************************/

    drv_open_device(*iod          ,    // I/O descriptor
                    (char*)taglist,    // Taglist          
                    3             ,    // Reqid
                    &ret           ) ; // Return code

    end : if (ret ) vhalt()          ; // OS halt if any error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_close-------------------------------------------------------

    Purpose : Closes and frees a device. If all the devices of the driver
              are closed, then terminates the driver.
*/

int myio_close(int iod)
  {
    int             ret              ; // Returned code
    int             driver, device   ; // Indexes in "iodrv[][]"
    int             list[2]          ; // Parameter for "del_uroute"

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that it is a value          *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]"                             *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect
    

/*****************************************************************************
 * Step 2 : Close the device. We have to call "drv_close_device" for "iod".  *
 *          This procedure sends a REQ_CLOSE request event to the driver     *
 *          VMIO automaton and here we unschedule until the corresponding    *
 *          RESP_CLOSE, sent by AUT_xyz, is received                         *
 *****************************************************************************/

    drv_close_device(iod      ,        // IOD of the device to be closed
                     4        ,        // Reqid
                     &ret      )     ; // Return code

    if ( ret ) goto end              ; // Exit if error


/*****************************************************************************
 * Step 3 : Free the device. We have to send to the AUT_xyz VMIO automaton a *
 * ------   REQ_FREE request event. We have to call the "drv_free" procedure *
 *          to do that. Then, we unschedule until the RESP_FREE response     *
 *          event is received. Then we clear the "iodrv[driver][defice]" IOD *
 *          value to indicate that device "device" of driver "driver" is now *
 *          free. Also, we decrement the count of allocated devices for that *
 *          driver, the "iodnb[driver]" global variable                      *
 *****************************************************************************/

    drv_free(iod      ,                // IOD of the device to be freed
             5        ,                // reqid
             &ret       )            ; // Return code

    if ( ret ) goto end              ; // Exit if error

    iodrv[driver][device] = 0        ; // Clear the IOD value
    iodnb[driver] --                 ; // Update the count of allocated devices


/*****************************************************************************
 * Step 4 : If the count of remaining allocated devices has reached 0, then  *
 * ------   we terminate the driver, we call the "drv_end" interface         *
 *          procedure that sends a REQ_END request event to the driver's     *
 *          VMIO  autiomaton and unschedule us until the RESP_END response   *
 *          event is received.                                               *
 *****************************************************************************/

    if ( iodnb[driver] > 0)            // If we have at least one remaining
      goto end                       ; // allocated device, do not terminate
      
    drv_end(modd[driver] ,             // MODD of the module to be ended
            6            ,             // Reqid
            &ret          )          ; // Return code

    modd[driver] = 0                 ; // Clear the MODD identifier

    if ( ret ) goto end              ; // Exit if error


/*****************************************************************************
 * Step 5 : Delete the route we were using for IND_REPORT or DRV_KEY events. *
 * -------  We have to call the "del_uroute" procedure. As this procedure    *
 *          takes as in put an array containing a list of route identifiers, *
 *          we use  here an array of one entry in which we store the         *
 *          "rteid[driver]" route identifier before callling "del_uroute"    *
 *          It has to be noted that we do not created any route with the     *
 *          LED and DAC driver and in suche a case "rteid[driver]" is 0      *
 *          ---------------------------------------------------------------- *
 *          We could directly call "del_route" with the address of the       *
 *          "asy_rteid" global variable because there is no difference       *
 *          between a variable adress and an array address                   *
 *****************************************************************************/

    if (rteid[driver] == 0) goto end ; // If no route has been created

    list[0] = rteid[driver]          ; // Store "asy_rteid" in a one entry
                                       // array
    del_uroute(list ,                  // We call "del_uroute" with a list
               1     )               ; // of route identifers tant only
                                       // contains one element.
    rteid[driver] = 0                ; // Clear the route identifier


    end : if ( ret ) vhalt()         ; // OS stop is error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_alloc_sub---------------------------------------------------

    Purpose : Allocates one sub-channel of one opened device (GPIO or LED)
*/

int myio_alloc_sub(int iod, const char *tagini, int *iodsub)
  {
    int             ret              ; // Returned code
    int             driver, device   ; // Indexes in "iodrv[][]"

/*****************************************************************************
 * Step 1 : Check that "iod" is a correct value, that it is a value          *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]"                             *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect
    

/*****************************************************************************
 * Step 2 : We are going to allocate one pin/led  subchannel. We call the    *
 * -------  "drv_alloc_subchan" procedure, this procedure sends a            *
 *          REQ_ALLOC_SUB event to the driver's VMIO automaton. This         *
 *          automaton will then send us a RESP_ALLOC_SUB response event.     *
 *          Here we unschedule (in "drv_alloc_subchan") until this event is  *
 *          received.                                                        *
 *****************************************************************************/

    drv_alloc_subchan(iod           ,  // I/O descriptor of controller
                      (char*)tagini ,  // Tagini  = PIN/LED selector
                      ""            ,  // Taglist = empty
                      0             ,  // Flags
                      7             ,  // Reqid
                      0             ,  // Route ID   = empty
                      0             ,  // Route ID 2 = empty
                      iodsub        ,  // Returned I/O descriptor for PIN/LED
                      &ret         ) ; // Return code

    end : if ( ret ) vhalt()         ; // OS stop is error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_free_sub ---------------------------------------------------

    Purpose : Frees one sub-channel of one opened device
*/

int myio_free_sub(int iodsub)
  {
    int             ret              ; // Returned code
    int             driver, device   ; // Indexes in "iodrv[][]"

/*****************************************************************************
 * Step 1 : Check that "iodsub|0xFF" is a correct value, that it is a value  *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iosub | 0xFF           *
 *****************************************************************************/

    ret = chk_iod(iodsub,              // Check that "iodsub | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect
    

/*****************************************************************************
 * Step 2 : Free the subchannel. We call "drv_free_subchan", this procedure  *
 * ------   sends a REQ_FREE_SUB request event to the driver's VMIO automaton*
 *          and here we unschedule until the corresponding RESP_FREE_SUB,    *
 *          sent by the driver's VMIO FSM is received.                       *
 *****************************************************************************/

    drv_free_subchan(iodsub    ,       // IOD of the subchannel to be freed
                     8         ,       // Reqid
                     &ret        )   ; // Return code

    end : if ( ret ) vhalt()         ; // OS stop is error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_getval -----------------------------------------------------

    Purpose : Reads, gets a taglist
*/

int myio_getval(int iod, char *taglist, int sz)
  {
    int             ret              ; // Returned code
    int             driver, device   ; // Indexes in "iodrv[][]"

/*****************************************************************************
 * Step 1 : Check that "iod|0xFF" is a correct value, that it is a value     *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iod | 0xFF              *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect


/*****************************************************************************
 * Step 2 : Read the taglist "taglist" from the driver. The "drv_getval"     *
 * ------   procedure checks the syntax of the "taglist" template string and *
 *          if there is an error it immediately returns with an error code.  *
 *          If the syntax is correct, a telecom buffer is allocated. If this *
 *          allocation fails, the procedure also returns immediately with an *
 *          error code. If the buffer allocation is a success, the           *
 *          compiled register list is put in the telecom buffer, that is     *
 *          pointed by a REQ_GETVAL event that is sent to the driver's VMIO  *
 *          automaton. The VMIO FSM fills the register list with the tags    *
 *          values to be read and returns a RESP_GETVAL that points to the   *
 *          modified register list. Then the returned register list is       *
 *          converted to a character string. The "drv_getval" unschedules    *
 *          our task, until the REST_GETVAL response event is received from  *
 *          the driver's FSM.                                                *
 *****************************************************************************/

    drv_getval(iod            ,        // I/O descriptor for DEV1
               (char*)taglist ,        // taglist buffer
               sz             ,        // taglist buffer sioze in bytes
               9              ,        // Reqid
               &ret             )    ; // Return code 

    end : if ( ret ) vhalt()         ; // OS stop if error

    return ret                       ; // Exit with "ret" as error code
  }

/*  Procedure myio_setval -----------------------------------------------------

    Purpose : writes, sets a taglist
*/

int myio_setval(int iod, const char *taglist)
  {
    int             ret              ; // Returned code
    int             driver, device   ; // Indexes in "iodrv[][]"

/*****************************************************************************
 * Step 1 : Check that "iod|0xFF" is a correct value, that it is a value     *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iod | 0xFF              *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect


/*****************************************************************************
 * Step 2 : Write the taglist "taglist" to the driver. The "drv_setval"      *
 * ------   procedure checks the syntax of the "taglist" string and if there *
 *          is an error it immediately returns with an error code. If the    *
 *          syntax is correct, a telecom buffer is allocated. If this        *
 *          allocation fails, the procedure also returns immediately with an *
 *          error code. If the buffer allocation is a success, the           *
 *          compiled register list is put in the telecom buffer, that is     *
 *          pointed by a REQ_SETVAL event that is sent to the driver's VMIO  *
 *          automaton. The "drv_setval" unschedules our task, until the      *
 *          REST_SETVAL response event is received from the driver's FSM.    *
 *****************************************************************************/

    drv_setval(iod            ,        // I/O descriptor for DEV1
               (char*)taglist ,        // taglist
               10             ,        // Reqid
               &ret             )    ; // Return code 

    end : if ( ret ) vhalt()         ; // OS stop if error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_givetok ----------------------------------------------------

    Purpose : Gives receive tokens to a driver
*/

int myio_givetok(int iod, int nbtok, int size, int *count)
  {
    int             ret              ; // Return code
    int             driver, device   ; // Indexes in "iodrv[][]"
    int             of7              ; // Firts byte in buffer (unused)
    int             lg               ; // Number of bytes read (unused)

/*****************************************************************************
 * Step 1 : Check that "iod|0xFF" is a correct value, that it is a value     *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iod | 0xFF              *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect


/*****************************************************************************
 * Step 2 : In order to have data flow control, the application have to      *
 * ------   give to the driver receive tokens. One token is one buffer.      *
 *          The applications gives to the driver a number of tokens "nbtok"  *
 *          and the size "size" of the buffers. The application as to send   *
 *          a REQ_READ request to the VMIO automaton. To allocate the "nbtok"*
 *          buffers and send one REQ_READ, we call "drv_read".               *
 *          ---------------------------------------------------------------- *
 *          RESP_READ event will be received after data will be received     *
 *          from the telecomunication line. This may take a long time, or    *
 *          may never occur. So we are not going to unschedule right now     *
 *****************************************************************************/

    drv_read(iod       ,               // I/O descriptor for DEV0
             size      ,               // Maximum number of bytes / buffer
             nbtok     ,               // token/buffer number
             0         ,               // Flags
             HNULL     ,               // Buffer address (unused)
             &of7      ,               // Offset (unused)
             &lg       ,               // Number of bytes read (unused)
             11        ,               // Reqid
             &ret        )           ; // Return code

    *count += nbtok                  ; // Update token counter

    end : if ( ret )                   // If an error has occured
            vhalt()                  ; // OS stop if error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_write ------------------------------------------------------

    Purpose : writes one buffer. This is a blocking procedure, we are 
              unscheduled until the end of transmission, despite the fact the
              driver has been opened in NBLOCKING_IO.
*/

int myio_write(int iod, unsigned char *buf, int lg)
  {
    int             ret              ; // Return code
    int             driver, device   ; // Indexes in "iodrv[][]"
    int             waitlist[1][3]   ; // Parameter of "waitevt_task

/*****************************************************************************
 * Step 1 : Check that "iod|0xFF" is a correct value, that it is a value     *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iod | 0xFF             *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect


/*****************************************************************************
 * Step 2 : Send using serial controller "iod" the "lg" bytes that are       *
 * ------   in telecom buffer ponted by "buf".  We call the "drv_write"      *
 *          procedure that sends a REQ_WRITE request event to the AUT_ASY    *
 *          VMIO automaton. 
 *****************************************************************************/

    drv_write(iod       ,              // IOD of serial controller DEV0
              buf       ,              // Buffer Address
              0         ,              // Buffer Offset
              lg        ,              // Number of bytes to be sent
              0         ,              // Flags
              12        ,              // Reqid
              &ret       )           ; // Return code

    if ( ret ) goto end              ; // Exit if error


/*****************************************************************************
 * Step 3 : We unschedule until the RESP_WRITE event is received. This event *
 * ------   is sent by the VMIO automaton only when the last byte as been    *
 *          sent and not before. We build a wait list containing a single    *
 *          pair (code=RESP_WRITE,reserve=iod). The task will be unschedule  *
 *          until an event corresponding to the one expected is received by  *
 *          the task. This wait may be infinite if the RESP_WRITE event is   *
 *          never received.                                                  *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODERES   ; // Test "code" and "reserve" fields
    waitlist[0][1]  = RESP_WRITE     ; // The awaited value for "code"
    waitlist[0][2]  = iod            ; // The awaited value for "reserve"

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // No maximum waiting time
                 0        ,            // Do not purge previous events
                 &ret       )        ; // Return code

    end : if ( ret ) vhalt()         ; // OS stop if error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure myio_send -------------------------------------------------------

    Purpose : Give one message to the driver. This procedure is non-blocking.
*/

int myio_send(int iod, unsigned char *buf, int lg, int *count)
  {
    int             ret              ; // Return code
    int             driver, device   ; // Indexes in "iodrv[][]"

/*****************************************************************************
 * Step 1 : Check that "iod|0xFF" is a correct value, that it is a value     *
 * ------   previously returned by the "drv_alloc_device" procedure. We call *
 *          the "chk_iod" subroutine, that also returns the two              *
 *          corresponding indexes in "iodrv[][]" for "iod | 0xFF             *
 *****************************************************************************/

    ret = chk_iod(iod,                 // Check that "iod | 0xFF" is one
                  &driver,&device)   ; // "drv_alloc_device" returned value

    if ( ret ) goto end              ; // Exit if "iod" is incorrect


/*****************************************************************************
 * Step 2 : Send using "iod" the "lg" bytes that are in telecom buffer       *
 * ------   pointed by "buf".  We call the "drv_write" procedure that sends  *
 *          a REQ_WRITE request event to the driver VMIO automaton. We want  *
 *          to maximize throughput, so we don't wait for data ta have been   *
 *          sent here. We therefore do not unschedule.                       *
 *****************************************************************************/

    drv_write(iod       ,              // IOD of serial controller DEV0
              buf       ,              // Buffer Address
              0         ,              // Buffer Offset
              lg        ,              // Number of bytes to be sent
              0         ,              // Flags
              13        ,              // Reqid
              &ret       )           ; // Return code

    *count += 1                      ; // Update number of inflight messages

    end : if ( ret ) vhalt()         ; // OS stop if error

    return ret                       ; // Exit with "ret" as error code
  }


/*  Procedure chk_iod ---------------------------------------------------------

    Purpose : Subroutine that checks that an IOD value corresponds to one that
              has been allocated, returned by "drv_alloc_device."
*/

static int chk_iod(int iod,int *driver,int *device)
  {
    int             ret              ; // Returned code
    int             i, j             ; // Loop counters

/*****************************************************************************
 * Step 1 : This procedure is allowed to receive sub-channel IOD's. We       *
 * ------   converts any sub-channel IOD into the corresponding device IOD   *
 *          by setting the 8 low-order bits to 1. If we have as input a      *
 *          device IOD, the 8 low-order bits are already 1's and setting     *
 *          those again is useless but harmless.                             *
 *****************************************************************************/

    iod |= 0x000000FF                ; // Force to a device IOD value


/*****************************************************************************
 * Step 2 : The "myio_open" procedures stores in the "iodrv[][]" static RAM  *
 * ------   table all the IOD's values that are returned by the              *
 *          "drv_alloc_device" procedure and the "myio_close" procedure      *
 *          clears those. Si here we loop on ll the entries of this array,   *
 *          looking for "iod". We have two loops since the array has two     *
 *          dimensions, the first is for drivers, "i" ranging from 0 to      *
 *          MAX_MODD-1 and the second is on devices, "j" ranging from 0 to   *
 *          MAX_DEV-1. We have two cases:                                    *
 *          - If we find the value, we exit from the loops and goto step2    *
 *          - If we do not find, we complete the loops and we return -1      *
 *            values for "ret", "driver" and "device"                        *
 *****************************************************************************/

    for (i = 0; i LT MAX_MODD; i++)    // We loop on all the entries of the
      for(j = 0; j LT MAX_DEV; j++)    // "iodrv[MAX_MODD][MAX_DEV]" array
        if (iodrv[i][j] == iod)        // If we find the "iod" value we are
          goto step2                 ; // looking for, exit from the loop

    ret     = -1                     ; // We failed, we did not found the
    *driver = -1                     ; // "iod" value, we set all the
    *device = -1                     ; // returned values to -1

    goto end                         ; // and we exit from the procedure


/*****************************************************************************
 * Step 3 : Here we have found "iod" in "iodrv[i]|j]". We set to 0 the       *
 * ------   function returned value and we set "driver" to "i" and "device"  *
 *          to "j".                                                          *
 *****************************************************************************/

    step2                            : // We found "iod"
    ret     = 0                      ; // No error
    *driver = i                      ; // The driver/module number
    *device = j                      ; // The device number

    end : if ( ret )                   // If we have an error, then
            vhalt()                  ; // OS 

    return ret                       ; // Exit with "ret" as error code
  }


